'3D振動応答解析V92の標準ﾓｼﾞｭｰﾙ        ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module FEM3VRCom
    '３次元構造の振動応答解析
    '「３次元構造解析」FEM3Dの構造データを利用
    'ＭＫ型とＭＣＫ型で振動外力への応答変化を計算
    'ＭＫ型：Ｋｘ＝λＭｘ
    'ＭＣＫ型：μ^2Ｍｚ＋μＣｚ＋Ｋｚ＝０

    'V81での改良点・・・反復法で節点間バネのバグを修正
    'V82での改良点・・・Visual Basic 2017に変更
    'V83での改良点・・・プログラムファイルの文字コードをUTF-8に統一、
    '                   データファイルの文字コードを標準でUTF-8に変更
    '                   （従来のShift-JISもオプションで可能）
    'V84での改良点・・・３次元グラフィック図で回転操作後の部分拡大の角度を保持
    'V85での改良点・・・構造図で座標位置荷重点が表示されないバグを修正
    'V86での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V87での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V88での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                   （これまでは最長方向の座標軸を採用）
    'V89での改良点・・・構造図画面で「隠面処理なし」の選択を廃止、
    '                   構造図画面で条件設定部の表示を２区分から３区分に改善
    'V90での改良点・・・四面体要素の廃止
    'V91での改良点・・・実行形式は.exeと.exe.configを同一フォルダに配置
    'V92での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$         '計算結果表示データ
    Public Cr$              'CR+LFのコード
    Public DirOpen$         '立ち上げ時のディレクトリー名
    Public Fgraph$          'ｸﾞﾗﾌﾃﾞｰﾀの出力ﾌｧｲﾙ名
    Public FLvib$           '振動外力のファイル名（旧）
    Public Fm$              '入力データのファイル名
    Public Fstruct$         'FEM3D構造ファイル名
    Public Fy$              '計算結果のファイル名
    Public Title$           'タイトル

    Public Timst As Date          '計算開始時刻

    Public InitCond As Integer      'ﾊﾞｯﾁ処理で計算条件設定の有無（0:なし、1:有り）
    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）
    Public KAnal As Integer         '解析種類（0:一般応答、1:周波数応答）
    Public Kboundfree As Integer    '境界拘束解除の有無（0:なし、1:有り）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public KcomSide As Integer      '共通辺の表示有無（0:なし、1:有り）
    Public KcontourPoint As Integer '等高線図で場所･数値表示の有無（0:なし、1:有り）
    Public KdataBackup As Integer   '入力データのバックアップファイル作成（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KdispENum As Integer     '要素番号の表示有無（0:なし、1:有り）
    Public KdispMat As Integer      '作画の材料の指定番号（0は全材料）
    Public KdispPNum As Integer     '節点番号の表示有無（0:なし、1:有り）
    Public KdispSNum As Integer     '面番号の表示有無（0:なし、1:有り）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public Kfinish As Integer       'ﾊﾞｯﾁ処理１回の終了区分（0:未終了、1:終了）
    Public KindBatch As Integer     'ﾊﾞｯﾁ処理の有無（0:なし、1:有り）
    Public KindDamp As Integer      '減衰設定区分（0:比例減衰、1:個別設定）
    Public KindElem As Integer      '要素種類（0:六面体20節点、1:四面体10節点）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindGraph As Integer     'ｸﾞﾗﾌ種類（1:１節点多変数、2:多節点１変数）
    Public KindGrav As Integer      '自重計算の有無（0:なし、1:有り）
    Public KindLvib As Integer      '振動外力の種類（0:力、1:加速度）
    Public KindPost As Integer      '後処理の種類（1:等高線図、2:変位図、
    '                                3:ｸﾞﾗﾌﾃﾞｰﾀの作成、4:ｱﾆﾒｰｼｮﾝ、
    '                                5:周波数分析、6:端数処理、7:主要値表示）
    Public KindRevF As Integer      '回転遠心力の区分（0:なし、1:X軸回り、
    Public KindSLoad As Integer     '静荷重の有無（0:なし、1:有り）
    Public KindSolver As Integer    'ｿﾙﾊﾞｰの種類（0:反復法(前処理なし)、1:反復法(前処理有り)、
    '                                2:ｽｶｲﾗｲﾝ法）
    Public KindVEqua As Integer     '方程式種類（0:ＭＫ型、1:ＭＣＫ型）
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KmouseSurf As Integer    'ﾏｳｽ指定の面番号
    Public KnewUFileA As Integer    '最近使用ﾌｧｲﾙAの更新有無
    Public KnewUFileB As Integer    '最近使用ﾌｧｲﾙBの更新有無
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public KrearTreat As Integer    '隠面処理の有無（0:なし、1:有り）
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public Kstep As Integer         '指定の時間ﾒｯｼｭ番号
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public KvalueG As Integer       'ｸﾞﾗﾌﾃﾞｰﾀの変数ｲﾝﾃﾞｯｸｽ
    Public Kversion As Integer      '結果ファイルのバージョン番号（1～2）
    Public Mprop As Integer         '材料データの成分数(=3)
    Public Nall As Integer          '変数の総数
    Public NboundP As Integer       '拘束節点数
    Public NboundS As Integer       '拘束面数
    Public Ndamp As Integer         'ダンパーの数
    Public Ndim As Integer          '１節点当りの変数の数(=3)
    Public NDisp As Integer         '等高線の分割数
    Public NdivCyc As Integer       '１周期分割数
    Public Nelem As Integer         '要素数
    Public Nforce As Integer        '荷重節点数
    Public NforceA As Integer       '座標位置荷重数
    Public Ngaus As Integer         'ガウス積分点数(=3)
    Public NKmax As Integer         '最大繰返し数
    Public NLvib As Integer         '振動外力の荷重個数
    Public Nmat As Integer          '材料データ数
    Public Norder As Integer        '正方行列の次数
    Public NPgraph As Integer       'ｸﾞﾗﾌ節点指定（節点番号または節点数）
    Public Npoint As Integer        '節点数
    Public NPvib As Integer         '振動外力節点数
    Public NSdraw As Integer        '作画面の数（隠面処理時）
    Public Nspring As Integer       'バネ支持個数
    Public Nstep As Integer         '出力時間間隔の倍数（1以上の整数）
    Public NstepF As Integer        '振動外力時間間隔の倍数（1以上の整数）
    Public Nstress As Integer       '応力成分の数（=6）
    Public Nsurf As Integer         '表面分布荷重数
    Public Nvar As Integer          '１要素当りの変数の総数（=60）
    Public NvibTime As Integer      '振動外力の時間メッシュ数

    Public AccGrav As Single        '重力加速度
    Public AngleN As Single         '周期内角度（°）
    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public CInc As Single           '最適化計算の増倍率（通常2～3）
    Public CoeffK As Single         '質量比例の減衰係数
    Public CoeffM As Single         '剛性比例の減衰係数
    Public CRed As Single           '最適化計算の低減率（通常0.5～0.1）
    Public DispRatio As Single      '変位作画の倍率
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public DTime As Single          '振動応答計算の時間間隔
    Public EpsCalc As Single        '収束判定誤差比率
    Public EpsL As Single           '要素領域の許容誤差
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public FreqLVib As Single       '振動外力の周波数
    Public HFactor As Single        '視点の高さ倍率（基準1）
    Public RevPS As Single          '回転数（単位時間当り）
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public TimeMax As Single        '打切り時間
    Public TimeNew As Single        '現在の計算時刻
    Public VdispMax As Single       '作画変数の最高値
    Public VdispMin As Single       '作画変数の最低値
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'モデルのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'モデルのZ軸回りの回転角（右ねじ正、°）

    Public UFileA(6) As String      '最近使用した構造ﾌｧｲﾙの保存配列
    Public UFileB(6) As String      '最近使用した解析ﾌｧｲﾙの保存配列

    Public JdirectP(,) As Integer   '節点拘束の各軸方向拘束有無(0:非拘束、1:拘束）
    Public JdirectS(,) As Integer   '面拘束の各軸方向拘束有無(0:非拘束、1:拘束）
    Public JLoadVib(,) As Integer   '節点振動外力の各軸方向の有無
    Public KBsurf(,) As Integer     '拘束面の要素・面番号
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public KdispMatR() As Integer   '計算行列の変数番号対応の変位方向
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KE3DCut(2) As Integer    '構造図の面切断の指定（座標軸番号と正負選択）
    Public KEorder() As Integer     '作画順の要素番号
    Public KinnerLine() As Integer  '内部辺の指定（0:なし、1:有り）
    Public KPdamp(,) As Integer     'ダンパーの2節点番号
    Public KPforce() As Integer     '節点荷重の節点番号
    Public KPgraph(20) As Integer   'ｸﾞﾗﾌ節点番号
    Public KPspring(,) As Integer   'バネ支持の節点番号
    Public KPsurf(,) As Integer     '表面分布荷重の要素・面番号
    Public KPvib() As Integer       '振動外力の節点番号
    Public KRcomp() As Integer      '計算順序節点の元ﾃﾞｰﾀ節点順序
    Public KSelem(,) As Integer     '各要素面の隠面指定（0:なし、1：有り、-1:視線外）
    Public KSorder() As Integer     '作画順の要素の面番号
    Public KsurfPoint(8) As Integer '面の8節点の節点順位
    Public Lcomp() As Integer       '全変数の縮小ﾏﾄﾘｯｸｽ変数番号
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NodeMatR() As Integer    '計算行列の変数番号対応の節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public NrestP() As Integer      '拘束の節点番号
    Public NumMat() As Integer      '各要素の材料種類番号

    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public CenterPos(3) As Single   '構造中心の座標
    Public Cmatrix() As Single      'Ｃ行列
    Public Cspring(,) As Single     'バネ定数
    Public DataDamp(,) As Single    'ダンパーの外部点座標と減衰係数
    Public DataFreq(,) As Single    '周波数外力の各ﾃﾞｰﾀ（振幅と位相差）
    Public DataMat(,) As Single     '各材料の物性値
    Public Dratio(3) As Single      '最適化計算の変数変化ΔＸの比率（＝ΔＸ／Ｘ）
    Public Ds(20), Dt(20), Du(20) As Single  '形状関数のξ偏微分、η偏微分、ζ偏微分
    Public Dx(20), Dy(20), Dz(20) As Single  '形状関数のx偏微分、y偏微分、z偏微分
    Public EpsX(3) As Single        '最適化計算の変数Ｘの最小変化量（絶対値）
    Public FLvibTime(,) As Single   '各時間毎の振動外力
    Public Force(,) As Single       '節点荷重値（各軸方向）
    Public ForceA(,) As Single      '座標位置荷重値（各軸方向）
    Public Fspring(,) As Single     '初期バネ力（各軸方向）
    Public GravMulti(3) As Single   '自重のX,Y,Z方向の倍率
    Public Pnorm() As Single        '表面の垂直方向分布荷重（単位面積当り）
    Public PosGaus(3) As Single     '積分点の位置
    Public RinvJ(3, 3) As Double   '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ
    Public RJmat(3, 3) As Double   'ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ
    Public RMmatrix() As Single     'Ｍ行列（対角要素のみ）
    Public RouMat() As Single       '各材料の密度
    Public Shape(20) As Single      '形状関数
    Public Tdisp(,) As Single       '結果の変位
    Public TransMat(4, 4) As Single 'モデルの合成アフィン変換行列
    Public Tu() As Single           'FEM計算の解
    Public VdirectP(,) As Single    '節点拘束の各変位方向拘束値
    Public VdirectS(,) As Single    '面拘束の各変位方向拘束値
    Public VE3DCut(2) As Single     '構造図の面切断の座標値と許容誤差
    Public WeightG(3) As Single     '積分点の重み
    Public X(), Y() As Single       '節点座標
    Public Xdata(), Ydata(), Zdata() As Single    '節点座標
    Public Xdraw(8), Ydraw(8) As Single   '色塗り多角形の座標
    Public XE(20), YE(20), ZE(20) As Single   '要素計算の配列
    Public XFA(), YFA(), ZFA() As Single    '座標位置荷重のX,Y座標
    Public XG(27), YG(27), ZG(27) As Single  'ｶﾞｳｽ点の座標
    Public Xmodel(), Ymodel(), Zmodel() As Single 'モデル座標における節点座標
    Public Xmodel2(), Ymodel2(), Zmodel2() As Single   'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xnew(), Ynew(), Znew() As Single   '変位入りの節点座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xscreen2(), Yscreen2() As Single   'スクリーンにおける節点座標
    Public Xvar(3) As Single        '最適化計算の各変数Ｘ（要素内座標ξ、η、ζ）
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）
    Public Xview2(), Yview2(), Zview2() As Single  '視点座標における節点座標（左手系）

    Sub AccDataCheck(ByRef kh As Integer)
        '加速度加振ﾃﾞｰﾀのﾁｪｯｸ
        'kh･･･ｴﾗｰ有無（0:なし、2:周波数加振が全要素有り）
        Dim i, ic, ie, j, k, nmax As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        kh = 0
        If KAnal = 1 Then
            kh = 2
            For ie = 1 To Nelem
                ic = 0
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    For i = 1 To NPvib
                        If k = KPvib(i) Then
                            ic = 1
                            Exit For
                        End If
                    Next i
                Next j
                If ic = 0 Then
                    kh = 0
                    Exit Sub
                End If
            Next ie
        End If

    End Sub

    Sub AdjustRatio(ByRef ratio As Single)
        '変位比率の調整
        Dim nb As Integer
        Dim b1, b2, std As Single

        std = 0.05
        b1 = std / ratio
        If b1 < 1.0 Then
            ratio = 1.0
        ElseIf b1 < 10.0 Then
            b2 = b1 * 10
            ratio = Int(b2) / 10.0
        Else    '上位2桁を採用
            b2 = b1
            nb = 0
            Do
                b2 = b2 / 10
                nb = nb + 1
                If b2 < 10 Then
                    Exit Do
                End If
            Loop
            b2 = Int(b2 * 10)
            If nb < 2 Then
                ratio = b2
            Else
                ratio = b2 * 10 ^ (nb - 1)
            End If
        End If

    End Sub

    Sub AxisCheck(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single, ByRef axmax() As Single, ByRef axmin() As Single, _
    ByRef xdb() As Single, ByRef ydb() As Single, ByRef zdb() As Single)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        'axmax(),axmin()･･･X,Y,Z座標の最大・最小値
        'xdb(),ydb(),zdb()･･･節点座標
        Dim ie, ii, ik, j, k, nmax As Integer
        Dim b1 As Single
        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        axmin(1) = xdb(k)
                        axmin(2) = ydb(k)
                        axmin(3) = zdb(k)
                        For ii = 1 To 3
                            axmax(ii) = axmin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = xdb(k)
                                Case 2
                                    b1 = ydb(k)
                                Case 3
                                    b1 = zdb(k)
                            End Select
                            If b1 < axmin(ii) Then
                                axmin(ii) = b1
                            ElseIf b1 > axmax(ii) Then
                                axmax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = axmax(1) - axmin(1)
        For ii = 2 To 3
            b1 = axmax(ii) - axmin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single, ByRef zbuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ie, kind, nb As Integer
        Dim b2, rlenb As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        Call AxisCheck(1, Nelem, rlenb, AxisMax, AxisMin, Xdata, Ydata, Zdata)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        b2 = rlenb * 0.00002
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

        For i = 1 To Npoint
            xbuf(i) = Xdata(i)
            ybuf(i) = Ydata(i)
            zbuf(i) = Zdata(i)
        Next i
        For i = 1 To Npoint
            If Xdata(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If Ydata(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
            If Zdata(i) <> 0 Then
                Call RoundTreat(kind, nb, zbuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub AxisCheckS(ByVal k1 As Integer, ByRef k2 As Integer, ByRef xmin As Single, _
    ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single, _
    ByRef xdb() As Single, ByRef ydb() As Single)
        'X･Y座標の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        'xdb(),ydb()･･･節点座標
        Dim i, ie, ik, j, nmax As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = xdb(i) : xmax = xmin
                        ymin = ydb(i) : ymax = ymin
                    Else
                        If xdb(i) < xmin Then
                            xmin = xdb(i)
                        ElseIf xdb(i) > xmax Then
                            xmax = xdb(i)
                        End If
                        If ydb(i) < ymin Then
                            ymin = ydb(i)
                        ElseIf ydb(i) > ymax Then
                            ymax = ydb(i)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisSet(ByRef tdb(,) As Single)
        '座標の設定
        Dim i As Integer
        For i = 1 To Npoint
            Xnew(i) = Xdata(i) + DispRatio * tdb(i, 1)
            Ynew(i) = Ydata(i) + DispRatio * tdb(i, 2)
            Znew(i) = Zdata(i) + DispRatio * tdb(i, 3)
        Next i
    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        'kcbuf()･･･元の節点の変数順序
        Dim i, ie, ii, j, jj, k, k1, k2, mx, nmax As Integer

        If KindElem = 0 Then
            nmax = 20
        End If
        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To nmax - 1
                i = NNelem(ie, ii)
                For jj = ii + 1 To nmax
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie
        If Nspring > 0 Then
            For j = 1 To Nspring
                k1 = KPspring(j, 1)
                k2 = KPspring(j, 2)
                If k2 > 0 Then
                    k = Math.Abs(kcbuf(k1) - kcbuf(k2))
                    If k > mx Then mx = k
                End If
            Next j
        End If
        If Ndamp > 0 Then
            For i = 1 To Ndamp
                k2 = KPdamp(i, 2)
                If k2 > 0 Then
                    k2 = kcbuf(k2)
                    k1 = kcbuf(KPdamp(i, 1))
                    k = Math.Abs(k1 - k2)
                    If k > mx Then mx = k
                End If
            Next i
        End If
        mb = Ndim * (mx + 1)

    End Sub

    Sub CalFunc(ByRef sf As Single, ByVal s As Single, _
    ByVal t As Single, ByVal u As Single, ByVal xc As Single, _
    ByVal yc As Single, ByVal zc As Single)
        '任意の関数式を定義して、その関数値を計算する。
        'sf･･･関数値
        's,t,u･･･要素の局所座標
        'xc,yc,zc･･･現在位置の座標
        Dim i, nmax As Integer
        Dim b1, b2, b3, xbb, ybb, zbb As Single
        If KindElem = 0 Then
            nmax = 20
            Call ShapeFunc(0, s, t, u)  '形状関数の計算
        End If
        xbb = 0
        ybb = 0
        zbb = 0
        For i = 1 To nmax
            xbb = xbb + Shape(i) * XE(i)
            ybb = ybb + Shape(i) * YE(i)
            zbb = zbb + Shape(i) * ZE(i)
        Next i
        b1 = xbb - xc
        b2 = ybb - yc
        b3 = zbb - zc
        sf = b1 * b1 + b2 * b2 + b3 * b3

    End Sub

    Sub CfileInputU(ByVal fxx$, ByRef kh As Integer, ByVal kstepx As Integer)
        '計算結果ﾌｧｲﾙの読込み
        'UTF-8･･･BOMなし
        'fxx$･･･計算結果ﾌｧｲﾙ名
        'kh･･･ﾃﾞｰﾀ有無（0:なし、1:有り）
        'kstepx･･･指定ｽﾃｯﾌﾟ番号（-1の場合は計算条件部分）
        Dim aa$
        Dim istep As Integer

        kh = 0
        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String
        If kstepx < 0 Then
            Do
                aa$ = sr.ReadLine
                If aa$ = "＜パラメーター条件＞" Then
                    kh = 1
                    Exit Do
                End If
            Loop
            If KAnal = 0 Then
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                TimeMax = dat(1)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                DTime = dat(1)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NstepF = dat(1)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Nstep = dat(1)
            Else
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NKmax = dat(1)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NdivCyc = dat(1)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                EpsCalc = dat(1)
            End If
        Else
            If KAnal = 0 Then
                Do
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    If dat(0) = "＜時間メッシュ番号、時間＞" Then
                        istep = dat(1)
                        TimeNew = dat(2)
                        If istep = kstepx Then
                            kh = 1
                            Exit Do
                        End If
                    End If
                Loop
            Else
                Do
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    If dat(0) = "＜周期内メッシュ番号、角度、周期内時間＞" Then
                        istep = dat(1)
                        AngleN = dat(2)
                        TimeNew = dat(3)
                        If istep = kstepx Then
                            kh = 1
                            Exit Do
                        End If
                    End If
                Loop
            End If
        End If

        sr.Close()

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Sub CosCalc2(ByVal aval1() As Single, ByVal aval2() As Single, _
        ByRef acos() As Single)
        '2点間の方向余弦の計算
        'aval1(),aval2()・・・2点のXYZ座標
        'acos()・・・XYZ座標への方向余弦
        Dim j As Integer
        Dim b1 As Single
        Dim buf(3) As Single

        For j = 1 To 3
            buf(j) = aval2(j) - aval1(j)
        Next j
        b1 = 0
        For j = 1 To 3
            b1 = b1 + buf(j) * buf(j)
        Next j
        b1 = Math.Sqrt(b1)
        For j = 1 To 3
            acos(j) = buf(j) / b1
        Next j

    End Sub

    Private Sub CrosPoint(ByRef alphab As Single, ByRef beta As Single, _
    ByVal eps As Single, ByVal xbuf() As Single, ByVal ybuf() As Single, _
    ByVal xbufc() As Single, ByVal ybufc() As Single)
        '2線分の交点計算
        'alphab,beta･･･各線分の交点位置比
        'xbuf(),ybuf()･･･基準側の2節点座標
        'xbufc(),ybufc()･･･比較側の2節点座標
        Dim a1, a2, b1, b2, b3, b4, bb, cc, dd, dist, distb As Single
        Dim eps2, f1, f2, xb, yb As Single

        alphab = -1
        eps2 = eps * eps
        a1 = ybuf(2) - ybuf(1)
        b1 = -(xbuf(2) - xbuf(1))
        dist = Math.Sqrt(a1 * a1 + b1 * b1)
        a2 = ybufc(2) - ybufc(1)
        b2 = -(xbufc(2) - xbufc(1))
        distb = Math.Sqrt(a2 * a2 + b2 * b2)
        If dist < eps Or distb < eps Then
            Exit Sub
        End If
        If Math.Abs(a1) < eps And Math.Abs(b2) < eps Then
            xb = xbufc(1)
            yb = ybuf(1)
            b1 = xb - xbuf(1)
            b2 = yb - ybufc(1)
            cc = b1 * (xb - xbuf(2))
            dd = b2 * (yb - ybufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        ElseIf Math.Abs(b1) < eps And Math.Abs(a2) < eps Then
            xb = xbuf(1)
            yb = ybufc(1)
            b1 = yb - ybuf(1)
            b2 = xb - xbufc(1)
            cc = b1 * (yb - ybuf(2))
            dd = b2 * (xb - xbufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        Else
            f1 = a1 * xbuf(1) + b1 * ybuf(1)
            f2 = a2 * xbufc(1) + b2 * ybufc(1)
            bb = a1 * b2 - b1 * a2
            If Math.Abs(bb) > eps2 Then
                xb = (f1 * b2 - f2 * b1) / bb
                yb = (f2 * a1 - f1 * a2) / bb
                b1 = xb - xbuf(1)
                b2 = yb - ybuf(1)
                cc = b1 * (xb - xbuf(2))
                dd = b2 * (yb - ybuf(2))
                If cc < eps2 And dd < eps2 Then
                    b3 = xb - xbufc(1)
                    b4 = yb - ybufc(1)
                    cc = b3 * (xb - xbufc(2))
                    dd = b4 * (yb - ybufc(2))
                    If cc < eps2 And dd < eps2 Then
                        alphab = Math.Sqrt(b1 * b1 + b2 * b2) / dist
                        beta = Math.Sqrt(b3 * b3 + b4 * b4) / distb
                    End If
                End If
            End If
        End If

    End Sub

    Sub DampErrorCheck(ByRef kh As Integer)  'ﾀﾞﾝﾊﾟｰのｴﾗｰﾁｪｯｸ
        'kh･･･ｴﾗｰ有無（0：なし、1：節点番号ｴﾗｰ、2：減衰係数ｴﾗｰ）
        Dim i, j, kb As Integer

        kh = 0
        For i = 1 To Ndamp
            For j = 1 To 2
                kb = KPdamp(i, j)
                If j = 1 Or kb > 0 Then
                    If kb <= 0 Or kb > Npoint Then
                        kh = 1
                        Exit Sub
                    End If
                End If
            Next j
            If DataDamp(i, 4) <= 0 Then
                kh = 2
                Exit Sub
            End If
        Next i

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub Dfile2InputU(ByVal fxx$)
        '構造ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, im, j, jb, jm, nmax As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "FEM3D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindElem = dat(1)
        Title$ = sr.ReadLine
        If KindElem = 0 Then
            nmax = 20
        Else
            aa$ = "六面体要素以外は使用できません！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NboundP = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NboundS = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nforce = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NforceA = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nsurf = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nspring = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindGrav = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindRevF = dat(1)
        If KindRevF > 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            RevPS = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            AccGrav = dat(1)
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ngaus = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Mprop = dat(1)

        ReDim NNelem(Nelem, nmax), NumMat(Nelem)
        ReDim Xdata(Npoint), Ydata(Npoint), Zdata(Npoint)
        ReDim DataMat(Nmat, Mprop)
        ReDim NrestP(NboundP), JdirectP(NboundP, Ndim), VdirectP(NboundP, Ndim)
        ReDim KBsurf(NboundS, 2)
        ReDim JdirectS(NboundS, Ndim), VdirectS(NboundS, Ndim)
        ReDim KPforce(Nforce), Force(Nforce, Ndim)
        ReDim ForceA(NforceA, Ndim), XFA(NforceA), YFA(NforceA), ZFA(NforceA)
        ReDim KPsurf(Nsurf, 2), Pnorm(Nsurf)
        ReDim KPspring(Nspring, 2), Cspring(Nspring, 3), Fspring(Nspring, 3)

        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 10
                NNelem(i, j) = dat(j + 1)
            Next j
            If nmax > 10 Then
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 11 To nmax
                    NNelem(i, j) = dat(j - 11)
                Next j
            End If
        Next i

        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Xdata(i) = dat(1)
            Ydata(i) = dat(2)
            Zdata(i) = dat(3)
        Next i

        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        For im = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For jm = 1 To Mprop
                DataMat(im, jm) = dat(jm)
            Next jm
        Next im

        If NboundP > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To NboundP
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NrestP(i) = dat(1)
                For j = 1 To Ndim
                    jb = 2 * j
                    JdirectP(i, j) = dat(jb)
                    VdirectP(i, j) = dat(jb + 1)
                Next j
            Next i
        End If

        If NboundS > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To NboundS
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KBsurf(i, 1) = dat(1)
                KBsurf(i, 2) = dat(2)
                For j = 1 To Ndim
                    jb = 2 * j
                    JdirectS(i, j) = dat(jb + 1)
                    VdirectS(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If Nforce > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To Nforce
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPforce(i) = dat(1)
                For j = 1 To Ndim
                    Force(i, j) = dat(j + 1)
                Next j
            Next i
        End If

        If NforceA > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To NforceA
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                XFA(i) = dat(1)
                YFA(i) = dat(2)
                ZFA(i) = dat(3)
                For j = 1 To Ndim
                    ForceA(i, j) = dat(j + 3)
                Next j
            Next i
        End If

        If Nsurf > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To Nsurf
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPsurf(i, 1) = dat(1)
                KPsurf(i, 2) = dat(2)
                Pnorm(i) = dat(3)
            Next i
        End If

        If Nspring > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To Nspring
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPspring(i, 1) = dat(1)
                KPspring(i, 2) = dat(2)
                For j = 1 To Ndim
                    jb = 2 * j
                    Cspring(i, j) = dat(jb + 1)
                    Fspring(i, j) = dat(jb + 2)
                Next j
            Next i
        End If

        If KindGrav > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For i = 1 To 3
                GravMulti(i) = dat(i - 1)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, id, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "FEM3VibRes" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Fstruct$ = dat(1)
        Title$ = sr.ReadLine

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindVEqua = dat(1)
        If KindVEqua = 1 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KindDamp = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Ndamp = dat(1)
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KAnal = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Kboundfree = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Norder = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nmat = dat(1)

        ReDim RouMat(Nmat)
        ReDim NodeMatR(Norder), KdispMatR(Norder)

        If Ndamp > 0 Then
            ReDim KPdamp(Ndamp, 2)
            ReDim DataDamp(Ndamp, 4)
            aa$ = sr.ReadLine
            For i = 1 To Ndamp
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KPdamp(i, 1) = dat(1)
                KPdamp(i, 2) = dat(2)
                For j = 1 To 4
                    DataDamp(i, j) = dat(j + 2)
                Next j
            Next i
        End If

        aa$ = sr.ReadLine
        For i = 1 To Nmat
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            RouMat(i) = dat(1)
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Norder
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NodeMatR(i) = dat(1)
            KdispMatR(i) = dat(2)
        Next i

        ReDim RMmatrix(Norder)
        aa$ = sr.ReadLine
        For i = 1 To Norder
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            RMmatrix(i) = dat(1)
        Next i

        If KindVEqua = 1 Then
            Select Case KindDamp
                Case 0
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    CoeffM = dat(1)
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    CoeffK = dat(1)
                Case 1
                    ReDim Cmatrix(Norder)
                    aa$ = sr.ReadLine
                    For i = 1 To Norder
                        aa$ = sr.ReadLine
                        dat = aa.Split(",")
                        Cmatrix(i) = dat(1)
                    Next i
            End Select
        End If

        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NPvib = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NLvib = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If KAnal = 0 Then
            NvibTime = dat(1)
            ReDim FLvibTime(NvibTime, NLvib)
        Else
            FreqLVib = dat(1)
            ReDim DataFreq(NLvib, 2)
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindLvib = dat(1)
        ReDim KPvib(NPvib), JLoadVib(NPvib, 3)

        aa$ = sr.ReadLine
        For i = 1 To NPvib
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KPvib(i) = dat(1)
            For id = 1 To Ndim
                JLoadVib(i, id) = dat(id + 1)
            Next id
        Next i

        If KAnal = 0 Then
            aa$ = sr.ReadLine
            For i = 0 To NvibTime
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 1 To NLvib
                    FLvibTime(i, j) = dat(j)
                Next j
            Next i
        Else
            aa$ = sr.ReadLine
            For i = 1 To NLvib
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                DataFreq(i, 1) = dat(1)
                DataFreq(i, 2) = dat(2)
            Next i
        End If

        sr.Close()

    End Sub

    Sub DfileOut(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, aa$, ad$
        Dim i, ib, id, ii, j, n As Integer

        a1$ = ""
        ii = FreeFile()
        FileOpen(ii, fxx$, OpenMode.Output)
        WriteLine(ii, "プログラム名", "FEM3VibRes")
        WriteLine(ii, "［FEM3D構造ﾌｧｲﾙ］", Fstruct$)
        WriteLine(ii, Title$)
        WriteLine(ii, "節点数", Npoint)
        aa$ = "方程式種類(0:MK型、1:MCK型)"
        WriteLine(ii, aa$, KindVEqua)
        If KindVEqua = 0 Then
            KindDamp = 0
            Ndamp = 0
        Else
            aa$ = "減衰設定区分(0:比例減衰、1:個別設定)"
            WriteLine(ii, aa$, KindDamp)
            WriteLine(ii, "ダンパー数", Ndamp)
        End If
        aa$ = "解析の種類（0:一般応答、1:周波数応答）"
        WriteLine(ii, aa$, KAnal)
        aa$ = "境界拘束の解除（0:なし、1:有り）"
        WriteLine(ii, aa$, Kboundfree)
        WriteLine(ii, "行列の次数", Norder)
        WriteLine(ii, "材料データ数", Nmat)

        If Ndamp > 0 Then
            aa$ = "＜ダンパーデータ＞No.、節点番号1、節点番号2、外部点X、Y、Z、減衰係数"
            WriteLine(ii, aa$)
            For i = 1 To Ndamp
                aa$ = Str$(i) + ",  " + Str$(KPdamp(i, 1)) + ",  " + Str$(KPdamp(i, 2))
                For j = 1 To 4
                    aa$ = aa$ + ",  " + Str$(DataDamp(i, j))
                Next j
                PrintLine(ii, aa$)
            Next i
        End If

        WriteLine(ii, "＜材料の密度＞材料番号、密度")
        For i = 1 To Nmat
            aa$ = Str$(i) + ",  " + Str$(RouMat(i))
            PrintLine(ii, aa$)
        Next i

        aa$ = "＜行列の変数番号対応＞変数番号、節点番号、変位方向"
        WriteLine(ii, aa$)
        For i = 1 To Norder
            aa$ = Str$(i) + ",  " + Str$(NodeMatR(i))
            aa$ = aa$ + ",  " + Str$(KdispMatR(i))
            PrintLine(ii, aa$)
        Next i

        WriteLine(ii, "＜Ｍ行列（質量マトリックス）＞行番号、各要素")
        For i = 1 To Norder
            aa$ = Str$(i) + ",  "
            aa$ = aa$ + Str$(RMmatrix(i))
            PrintLine(ii, aa$)
        Next i

        If KindVEqua = 1 Then
            Select Case KindDamp
                Case 0
                    WriteLine(ii, "質量比例の減衰係数", CoeffM)
                    WriteLine(ii, "剛性比例の減衰係数", CoeffK)
                Case 1
                    WriteLine(ii, "＜Ｃ行列（減衰マトリックス）＞行番号、各要素")
                    For i = 1 To Norder
                        aa$ = Str$(i) + ",  "
                        aa$ = aa$ + Str$(Cmatrix(i))
                        PrintLine(ii, aa$)
                    Next i
            End Select
        End If

        ad$ = "-------------------------------------------------------"
        WriteLine(ii, ad$)
        WriteLine(ii, "＜振動外力データ＞")
        WriteLine(ii, "振動外力の節点数", NPvib)
        WriteLine(ii, "振動外力の荷重個数", NLvib)
        If KAnal = 0 Then
            WriteLine(ii, "振動外力の時間メッシュ数", NvibTime)
        Else
            WriteLine(ii, "振動外力の周波数", FreqLVib)
        End If
        WriteLine(ii, "振動外力の種類（0:力、1:加速度）", KindLvib)
        aa$ = "No.、節点番号、X方向有無、Y方向有無、Z方向有無"
        WriteLine(ii, aa$)
        For i = 1 To NPvib
            aa$ = Str$(i) + ",  " + Str$(KPvib(i))
            For id = 1 To Ndim
                aa$ = aa$ + ",   " + Str$(JLoadVib(i, id))
            Next id
            PrintLine(ii, aa$)
        Next i
        If KAnal = 0 Then
            aa$ = "時間No."
            For i = 1 To NPvib
                ib = KPvib(i)
                For id = 1 To 3
                    j = JLoadVib(i, id)
                    If j > 0 Then
                        Select Case id
                            Case 1
                                a1$ = "u"
                            Case 2
                                a1$ = "v"
                            Case 3
                                a1$ = "w"
                        End Select
                        a1$ = a1$ + LTrim$(Str$(ib))
                        aa$ = aa$ + "、" + a1$
                    End If
                Next id
            Next i
            WriteLine(ii, aa$)
            For i = 0 To NvibTime
                aa$ = Str$(i)
                For j = 1 To NLvib
                    aa$ = aa$ + ",  " + Str$(FLvibTime(i, j))
                Next j
                PrintLine(ii, aa$)
            Next i
        Else
            aa$ = "変位方向、振幅、位相差"
            WriteLine(ii, aa$)
            n = 0
            For i = 1 To NPvib
                ib = KPvib(i)
                For id = 1 To 3
                    j = JLoadVib(i, id)
                    If j > 0 Then
                        n = n + 1
                        Select Case id
                            Case 1
                                a1$ = "u"
                            Case 2
                                a1$ = "v"
                            Case 3
                                a1$ = "w"
                        End Select
                        aa$ = a1$ + LTrim$(Str$(ib))
                        aa$ = aa$ + ",  " + Str(DataFreq(n, 1))
                        aa$ = aa$ + ",  " + Str(DataFreq(n, 2))
                        PrintLine(ii, aa$)
                    End If
                Next id
            Next i
        End If

        FileClose(ii)

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, aa$, ad$
        Dim i, ib, id, j, n As Integer

        a1$ = ""
        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "FEM3VibRes"
        sw.WriteLine(aa$)
        aa$ = "［FEM3D構造ﾌｧｲﾙ］" + "," + Fstruct$
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "方程式種類(0:MK型、1:MCK型)" + "," + Str(KindVEqua)
        sw.WriteLine(aa$)
        If KindVEqua = 0 Then
            KindDamp = 0
            Ndamp = 0
        Else
            aa$ = "減衰設定区分(0:比例減衰、1:個別設定)" + "," + Str(KindDamp)
            sw.WriteLine(aa$)
            aa$ = "ダンパー数" + "," + Str(Ndamp)
            sw.WriteLine(aa$)
        End If
        aa$ = "解析の種類（0:一般応答、1:周波数応答）" + "," + Str(KAnal)
        sw.WriteLine(aa$)
        aa$ = "境界拘束の解除（0:なし、1:有り）" + "," + Str(Kboundfree)
        sw.WriteLine(aa$)
        aa$ = "行列の次数" + "," + Str(Norder)
        sw.WriteLine(aa$)
        aa$ = "材料データ数" + "," + Str(Nmat)
        sw.WriteLine(aa$)

        If Ndamp > 0 Then
            aa$ = "＜ダンパーデータ＞No.、節点番号1、節点番号2、外部点X、Y、Z、減衰係数"
            sw.WriteLine(aa$)
            For i = 1 To Ndamp
                aa$ = Str$(i) + ",  " + Str$(KPdamp(i, 1)) + ",  " + Str$(KPdamp(i, 2))
                For j = 1 To 4
                    aa$ = aa$ + ",  " + Str$(DataDamp(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        End If

        sw.WriteLine("＜材料の密度＞材料番号、密度")
        For i = 1 To Nmat
            aa$ = Str$(i) + ",  " + Str$(RouMat(i))
            sw.WriteLine(aa$)
        Next i

        aa$ = "＜行列の変数番号対応＞変数番号、節点番号、変位方向"
        sw.WriteLine(aa$)
        For i = 1 To Norder
            aa$ = Str$(i) + ",  " + Str$(NodeMatR(i))
            aa$ = aa$ + ",  " + Str$(KdispMatR(i))
            sw.WriteLine(aa$)
        Next i

        sw.WriteLine("＜Ｍ行列（質量マトリックス）＞行番号、各要素")
        For i = 1 To Norder
            aa$ = Str$(i) + ",  "
            aa$ = aa$ + Str$(RMmatrix(i))
            sw.WriteLine(aa$)
        Next i

        If KindVEqua = 1 Then
            Select Case KindDamp
                Case 0
                    aa$ = "質量比例の減衰係数" + "," + Str(CoeffM)
                    sw.WriteLine(aa$)
                    aa$ = "剛性比例の減衰係数" + "," + Str(CoeffK)
                    sw.WriteLine(aa$)
                Case 1
                    sw.WriteLine("＜Ｃ行列（減衰マトリックス）＞行番号、各要素")
                    For i = 1 To Norder
                        aa$ = Str$(i) + ",  "
                        aa$ = aa$ + Str$(Cmatrix(i))
                        sw.WriteLine(aa$)
                    Next i
            End Select
        End If

        ad$ = "-------------------------------------------------------"
        sw.WriteLine(ad$)
        sw.WriteLine("＜振動外力データ＞")
        aa$ = "振動外力の節点数" + "," + Str(NPvib)
        sw.WriteLine(aa$)
        aa$ = "振動外力の荷重個数" + "," + Str(NLvib)
        sw.WriteLine(aa$)
        If KAnal = 0 Then
            aa$ = "振動外力の時間メッシュ数" + "," + Str(NvibTime)
        Else
            aa$ = "振動外力の周波数" + "," + Str(FreqLVib)
        End If
        sw.WriteLine(aa$)
        aa$ = "振動外力の種類（0:力、1:加速度）" + "," + Str(KindLvib)
        sw.WriteLine(aa$)
        aa$ = "No.、節点番号、X方向有無、Y方向有無、Z方向有無"
        sw.WriteLine(aa$)
        For i = 1 To NPvib
            aa$ = Str$(i) + ",  " + Str$(KPvib(i))
            For id = 1 To Ndim
                aa$ = aa$ + ",   " + Str$(JLoadVib(i, id))
            Next id
            sw.WriteLine(aa$)
        Next i
        If KAnal = 0 Then
            aa$ = "時間No."
            For i = 1 To NPvib
                ib = KPvib(i)
                For id = 1 To 3
                    j = JLoadVib(i, id)
                    If j > 0 Then
                        Select Case id
                            Case 1
                                a1$ = "u"
                            Case 2
                                a1$ = "v"
                            Case 3
                                a1$ = "w"
                        End Select
                        a1$ = a1$ + LTrim$(Str$(ib))
                        aa$ = aa$ + "、" + a1$
                    End If
                Next id
            Next i
            sw.WriteLine(aa$)
            For i = 0 To NvibTime
                aa$ = Str$(i)
                For j = 1 To NLvib
                    aa$ = aa$ + ",  " + Str$(FLvibTime(i, j))
                Next j
                sw.WriteLine(aa$)
            Next i
        Else
            aa$ = "変位方向、振幅、位相差"
            sw.WriteLine(aa$)
            n = 0
            For i = 1 To NPvib
                ib = KPvib(i)
                For id = 1 To 3
                    j = JLoadVib(i, id)
                    If j > 0 Then
                        n = n + 1
                        Select Case id
                            Case 1
                                a1$ = "u"
                            Case 2
                                a1$ = "v"
                            Case 3
                                a1$ = "w"
                        End Select
                        aa$ = a1$ + LTrim$(Str$(ib))
                        aa$ = aa$ + ",  " + Str(DataFreq(n, 1))
                        aa$ = aa$ + ",  " + Str(DataFreq(n, 2))
                        sw.WriteLine(aa$)
                    End If
                Next id
            Next i
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Function DispMax(ByVal k1 As Integer, ByRef k2 As Integer, _
    ByRef tdb(,) As Single) As Single
        '変位の最大値
        'k1,k2･･･要素の開始、終了番号
        'dmax･･･X座標最小値
        'tdb(,)･･･節点の各座標方向変位
        Dim i, ie, j, k, nmax As Integer
        Dim b1, dmax As Single
        If KindElem = 0 Then
            nmax = 20
        End If
        dmax = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    i = NNelem(ie, j)
                    For k = 1 To 3
                        b1 = Math.Abs(tdb(i, k))
                        If b1 > dmax Then
                            dmax = b1
                        End If
                    Next k
                Next j
            End If
        Next ie
        Return dmax

    End Function

    Sub DispRateSet(ByRef ratio As Single, ByVal tfdb(,) As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'tfdb(,)･･･節点の各座標変位
        Dim rlen, rlen2 As Single

        Call AxisCheck(KE1, KE2, rlen, AxisMax, AxisMin, Xdata, Ydata, Zdata)
        '座標の最小・最大値のﾁｪｯｸ

        rlen2 = DispMax(KE1, KE2, tfdb)
        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Sub DispRateAnim(ByRef ratio As Single, ByVal rlen2 As Single)
        '変位作画倍率の設定
        'ratio･･･変位作画の倍率
        'rlen2･･･最大変位
        Dim rlen As Single

        Call AxisCheck(KE1, KE2, rlen, AxisMax, AxisMin, Xdata, Ydata, Zdata)
        '座標の最小・最大値のﾁｪｯｸ

        ratio = rlen2 / rlen
        Call AdjustRatio(ratio)     '変位比率の調整

    End Sub

    Sub DistComp(ByRef ic As Integer, ByVal kbuf() As Integer)
        '距離比較
        'ic･･･基準面の距離区分（0:近い、1:遠い）
        'kbuf()･･･比較面の8節点番号
        Dim i, i1, i2, ib, icc, j, j1, j2, jb, nmax As Integer
        Dim alphab, b1, b2, b3, beta, c1, dist, distc, eps, xb, yb, zb As Single
        Dim xbuf(2), xbufc(2), ybuf(2), ybufc(2) As Single

        If KindElem = 0 Then
            nmax = 8
            c1 = 0.25
        End If
        ic = -1  '仮設定
        For i = 1 To nmax - 1 Step 2
            i1 = KsurfPoint(i)
            ib = i + 2
            If ib > nmax Then
                ib = 1
            End If
            i2 = KsurfPoint(ib)
            xbuf(1) = Xscreen(i1)
            ybuf(1) = Yscreen(i1)
            xbuf(2) = Xscreen(i2)
            ybuf(2) = Yscreen(i2)
            b1 = Xmodel(i1) - Xmodel(i2)
            b2 = Ymodel(i1) - Ymodel(i2)
            b3 = Zmodel(i1) - Zmodel(i2)
            eps = 0.0001 * Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
            For j = 1 To nmax - 1 Step 2
                j1 = kbuf(j)
                jb = j + 2
                If jb > nmax Then
                    jb = 1
                End If
                j2 = kbuf(jb)
                icc = 0
                If i1 = j1 Or i1 = j2 Then
                    icc = 1
                ElseIf i2 = j1 Or i2 = j2 Then
                    icc = 1
                End If
                If icc = 0 Then
                    xbufc(1) = Xscreen(j1)
                    ybufc(1) = Yscreen(j1)
                    xbufc(2) = Xscreen(j2)
                    ybufc(2) = Yscreen(j2)
                    Call CrosPoint(alphab, beta, eps, xbuf, ybuf, xbufc, ybufc)
                    '2線分の交点計算

                    If alphab >= 0 Then
                        b1 = 1 - alphab
                        xb = b1 * Xmodel(i1) + alphab * Xmodel(i2)
                        yb = b1 * Ymodel(i1) + alphab * Ymodel(i2) + Rlenmodel
                        zb = b1 * Zmodel(i1) + alphab * Zmodel(i2) - Eyeh
                        dist = xb * xb + yb * yb + zb * zb
                        b1 = 1 - beta
                        xb = b1 * Xmodel(j1) + beta * Xmodel(j2)
                        yb = b1 * Ymodel(j1) + beta * Ymodel(j2) + Rlenmodel
                        zb = b1 * Zmodel(j1) + beta * Zmodel(j2) - Eyeh
                        distc = xb * xb + yb * yb + zb * zb
                        If dist < distc Then
                            ic = 0
                        Else
                            ic = 1
                        End If
                        icc = -1
                        Exit For
                    End If
                End If
            Next j
            If icc < 0 Then
                Exit For
            End If
        Next i

        If ic < 0 Then
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = KsurfPoint(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            dist = xb * xb + yb * yb + zb * zb
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = kbuf(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            distc = xb * xb + yb * yb + zb * zb

            If dist < distc Then
                ic = 0
            Else
                ic = 1
            End If
        End If

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawNodeVar(ByRef gr As Graphics, ByVal aa As String, _
    ByVal xb As Single, ByVal yb As Single)
        '節点変数の描画
        'aa･･･変数のテキスト
        'xb,yb･･･描画位置
        Dim xbw, ybw As Single
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", 10, FontStyle.Bold)

        xbw = xb - 1
        ybw = yb
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb
        ybw = yb - 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        ybw = yb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        gr.DrawString(aa$, fnt2, Brushes.Purple, xb, yb)

    End Sub

    Sub DrawScale(ByVal dpixb As Single, ByVal dpiyb As Single, _
    ByVal xlen As Single, ByVal ylen As Single, ByVal xmin As Single, _
    ByVal xmax As Single, ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        If Cfix < 1 Then
            If b1 <> 0 Then
                cx = xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2 / Cfix
            End If
        Else
            If b1 <> 0 Then
                cx = Cfix * xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2
            End If
        End If
        If b1 = 0 Then
            XYscale = cy
        ElseIf b2 = 0 Then
            XYscale = cx
        ElseIf cy < cx Then
            XYscale = cy
        Else
            XYscale = cx
        End If

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        KnumDisp = 0
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        KrearTreat = 1
        KdispENum = 0
        KdispPNum = 0
        KdispSNum = 0
        KdispMat = 0

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Sub ElemCheck(ByRef ic As Integer, ByVal ie As Integer, _
    ByVal klevel As Integer, ByRef s As Single, ByRef t As Single, _
    ByRef u As Single, ByVal xb As Single, ByVal yb As Single, ByVal zb As Single)
        '要素内点のﾁｪｯｸ
        'ic･･･要素内外の判定（0：外、1：内）
        'ie･･･要素番号
        'klevel･･･ﾁｪｯｸのﾚﾍﾞﾙ（0、1）
        's,t,u,･･･要素の局所座標
        'xb,yb,zb･･･点の座標
        Dim icc, j, k, kb, nmax As Integer
        Dim s0, sfm, xemax, xemin, yemax, yemin, zemax, zemin As Single
        Dim buf(1), cbuf(1) As Single
        buf(0) = 1.001
        buf(1) = 1.05
        cbuf(0) = -0.001
        cbuf(1) = -0.05
        If KindElem = 0 Then
            nmax = 20
        End If
        ic = 0
        For j = 1 To nmax
            kb = NNelem(ie, j)
            XE(j) = Xdata(kb)
            YE(j) = Ydata(kb)
            ZE(j) = Zdata(kb)
            If j = 1 Then
                xemin = XE(1)
                xemax = xemin
                yemin = YE(1)
                yemax = yemin
                zemin = ZE(1)
                zemax = zemin
            Else
                If XE(j) < xemin Then
                    xemin = XE(j)
                ElseIf XE(j) > xemax Then
                    xemax = XE(j)
                End If
                If YE(j) < yemin Then
                    yemin = YE(j)
                ElseIf YE(j) > yemax Then
                    yemax = YE(j)
                End If
                If ZE(j) < zemin Then
                    zemin = ZE(j)
                ElseIf ZE(j) > zemax Then
                    zemax = ZE(j)
                End If
            End If
        Next j
        icc = 0
        If xb < xemin - EpsL Then
            icc = 1
        ElseIf xb > xemax + EpsL Then
            icc = 1
        End If
        If icc = 0 Then
            If yb < yemin - EpsL Then
                icc = 1
            ElseIf yb > yemax + EpsL Then
                icc = 1
            End If
        End If
        If icc = 0 Then
            If zb < zemin - EpsL Then
                icc = 1
            ElseIf zb > zemax + EpsL Then
                icc = 1
            End If
        End If
        If icc = 0 Then
            s = 0.2 : t = 0.2 : u = 0.2
            Call KHSolver(k, 100, s0, sfm, s, t, u, xb, yb, zb)
            'KH法による最適化ｿﾙﾊﾞｰ
            If KindElem = 0 Then
                If s > -buf(klevel) And s < buf(klevel) Then
                    If t > -buf(klevel) And t < buf(klevel) Then
                        If u > -buf(klevel) And u < buf(klevel) Then
                            ic = 1
                        End If
                    End If
                End If
            End If
        End If

    End Sub

    Sub ElemFA(ByRef kelem As Integer, ByRef s As Single, _
        ByRef t As Single, ByRef u As Single, ByVal xi As Single, _
        ByVal yi As Single, ByVal zi As Single)
        '該当要素の探索
        'kelem･･･該当要素番号
        's,t,u･･･要素の局所座標（ξ、η、ζ）
        'xi,yi,zi･･･点の座標
        Dim ic, ie, k, klevel As Integer

        kelem = 0
        For k = 1 To 2
            If k = 1 Then
                klevel = 0
            Else
                klevel = 1
            End If
            For ie = 1 To Nelem
                Call ElemCheck(ic, ie, klevel, s, t, u, xi, yi, zi) '要素内点のﾁｪｯｸ
                If ic > 0 Then
                    kelem = ie
                    Exit For
                End If
            Next ie
            If kelem > 0 Then
                Exit For
            End If
        Next k

    End Sub

    Sub ElemRelation(ByRef icc As Integer, ByVal ie As Integer, ByVal ib As Integer)
        '要素関係の判別
        'icc･･･関係の有無（0:なし、1:有り）
        'ie･･･元の要素番号
        'ib･･･対象の要素番号
        Dim i, j, k As Integer
        If KindElem = 0 Then
            For i = 1 To 7 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
            For i = 13 To 19 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
        End If

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer, _
    ByRef xsb() As Single, ByRef ysb() As Single)
        '拡大作画する要素の探索
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        Dim ic, ie, j, k, kmin, kmax, nmax As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    b1 = xsb(k) - Xmouse(1)
                    b2 = xsb(k) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = ysb(k) - Ymouse(2)
                        b2 = ysb(k) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = nmax Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Sub EyeCheck(ByVal k1 As Integer, ByVal k2 As Integer)
        '視線ﾁｪｯｸ（面の中心位置で判定）
        'k1,k2･･･開始・終了要素番号
        Dim ie, j, k, kb, kmax, nb, nmax As Integer
        Dim b1, b2, b3, b4, xb, yb, zb As Single
        Dim eyevec(3), planevec(3) As Single
        Dim xbufb(3), ybufb(3), zbuf(3) As Single

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
            nb = 4
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If

                        xb = 0 : yb = 0 : zb = 0
                        For j = 2 To nmax Step 2
                            kb = KsurfPoint(j)
                            xb = xb + Xmodel(kb)
                            yb = yb + Ymodel(kb)
                            zb = zb + Zmodel(kb) - Eyeh
                            Select Case j
                                Case 2
                                    xbufb(2) = Xmodel(kb)
                                    ybufb(2) = Ymodel(kb)
                                    zbuf(2) = Zmodel(kb) - Eyeh
                                Case 4
                                    xbufb(3) = Xmodel(kb)
                                    ybufb(3) = Ymodel(kb)
                                    zbuf(3) = Zmodel(kb) - Eyeh
                            End Select
                        Next j
                        xbufb(1) = xb / nb
                        ybufb(1) = yb / nb
                        zbuf(1) = zb / nb

                        '逆視線ﾍﾞｸﾄﾙの計算
                        b1 = -xbufb(1)
                        b2 = -(Rlenmodel + ybufb(1))
                        b3 = -zbuf(1)
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        eyevec(1) = b1 / b4
                        eyevec(2) = b2 / b4
                        eyevec(3) = b3 / b4

                        '面の法線ﾍﾞｸﾄﾙの計算
                        b1 = (ybufb(2) - ybufb(1)) * (zbuf(3) - zbuf(1))
                        b1 = b1 - (zbuf(2) - zbuf(1)) * (ybufb(3) - ybufb(1))
                        b2 = (zbuf(2) - zbuf(1)) * (xbufb(3) - xbufb(1))
                        b2 = b2 - (xbufb(2) - xbufb(1)) * (zbuf(3) - zbuf(1))
                        b3 = (xbufb(2) - xbufb(1)) * (ybufb(3) - ybufb(1))
                        b3 = b3 - (ybufb(2) - ybufb(1)) * (xbufb(3) - xbufb(1))
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        planevec(1) = b1 / b4
                        planevec(2) = b2 / b4
                        planevec(3) = b3 / b4

                        '逆視線ﾍﾞｸﾄﾙと面法線ﾍﾞｸﾄﾙのｽｶﾗｰ積計算
                        b1 = 0
                        For j = 1 To 3
                            b1 = b1 + eyevec(j) * planevec(j)
                        Next j
                        If b1 <= 0 Then
                            KSelem(ie, k) = -1   '視線外
                        End If
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Function FreeMemoryGet() As Single
        '空きﾒﾓﾘｰ容量の取得
        'freemem･･･ﾒﾓﾘｰ空き容量（KB）
        Dim memfree As New System.Diagnostics.PerformanceCounter

        memfree.CategoryName = "Memory"
        memfree.CounterName = "Available KBytes"
        Return memfree.NextValue()

    End Function

    Sub InnerCheck2(ByVal k1 As Integer, ByVal k2 As Integer)
        '内面ﾁｪｯｸ
        'k1,k2･･･開始・終了要素番号
        Dim ib, ic, icc, ie, ik, ik1, j, jj, jx, k, kk, kmax, ksb, ksmax, m, nmax As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            ksmax = 4
            nmax = 7
            ReDim KinnerLine(12 * Nelem) '内部辺のﾌﾗｯｸﾞ
        End If
        Dim ksbuf(Nelem, kmax) As Integer    'ﾁｪｯｸ済みﾌﾗｯｸﾞ
        For ie = k1 To k2 - 1
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If k = 1 Then
                        icc = 0
                    End If
                    If ksbuf(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If
                        ik = 0
                        For ib = ie + 1 To k2
                            If KdrawElem(ib) > 0 Then
                                If icc = 0 Then
                                    Call ElemRelation(icc, ie, ib)    '要素関係の判別
                                End If
                                If icc > 0 Then
                                    For kk = 1 To kmax
                                        If ksbuf(ib, kk) = 0 Then
                                            If KindElem = 0 Then
                                                Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                            End If

                                            ik1 = KsurfPoint(1)
                                            ic = 0
                                            For j = 1 To nmax Step 2
                                                If ik1 = kbuf(j) Then
                                                    jx = j
                                                    For jj = 3 To nmax Step 2
                                                        jx = jx - 2
                                                        If jx < 1 Then
                                                            jx = nmax
                                                        End If
                                                        If KsurfPoint(jj) <> kbuf(jx) Then
                                                            ic = 1
                                                            Exit For
                                                        End If
                                                    Next jj
                                                    If ic = 0 Then
                                                        ik = 1
                                                        KSelem(ie, k) = 1
                                                        KSelem(ib, kk) = 1
                                                        ksbuf(ie, k) = 1
                                                        ksbuf(ib, kk) = 1
                                                        For m = 1 To ksmax
                                                            ksb = KSideLine(ie, k, m)
                                                            KinnerLine(ksb) = 1
                                                            ksb = KSideLine(ib, kk, m)
                                                            KinnerLine(ksb) = 1
                                                        Next m
                                                    End If
                                                    Exit For
                                                End If
                                            Next j
                                            If ik = 1 Then
                                                Exit For
                                            End If
                                        End If
                                    Next kk
                                    If ik = 1 Then
                                        Exit For
                                    End If
                                End If
                            Else
                                icc = 0
                            End If
                        Next ib
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub JacobElem(ByVal ie As Integer, ByVal kgaus As Integer, _
    ByVal numg As Integer, ByRef dj As Double)
        '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ie･･･要素番号
        'kgaus･･･ｶﾞｳｽ点座標計算の有無（0:なし、1:有り）
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        Dim c1$, c2$
        Dim i, ii, j, nmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            nmax = 20
        End If

        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        If kgaus > 0 Then
            XG(numg) = 0
            YG(numg) = 0
            ZG(numg) = 0
            For ii = 1 To nmax
                XG(numg) = XG(numg) + XE(ii) * Shape(ii)
                YG(numg) = YG(numg) + YE(ii) * Shape(ii)
                ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
            Next ii
        End If
        For ii = 1 To nmax
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ii
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)
        b1 = RJmat(1, 1) * RinvJ(1, 1) + RJmat(1, 2) * RinvJ(2, 1)
        dj = b1 + RJmat(1, 3) * RinvJ(3, 1)
        If dj <= 0 Then
            c1$ = "計算実行エラー"
            c2$ = "要素" + LTrim$(Str$(ie)) + "（ガウス点" + LTrim$(Str$(numg))
            c2$ = c2$ + "）でヤコビアン体積が負か０になりました。"
            MsgBox(c2$, 0, c1$)
            End
        Else
            '    本来の逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽは下記のようにdjで割った値である。
            '        For i = 1 To 3
            '            For j = 1 To 3
            '                RinvJ(i, j) = RinvJ(i, j) / dj
            '            Next j
            '        Next i
            For ii = 1 To nmax
                For i = 1 To 3
                    b1 = RinvJ(i, 1) * Ds(ii) + RinvJ(i, 2) * Dt(ii)
                    b1 = (b1 + RinvJ(i, 3) * Du(ii)) / dj
                    Select Case i
                        Case 1
                            Dx(ii) = b1
                        Case 2
                            Dy(ii) = b1
                        Case 3
                            Dz(ii) = b1
                    End Select
                Next i
            Next ii
        End If

    End Sub

    Sub JacobElem2(ByVal ie As Integer, ByVal numg As Integer, _
    ByRef dj As Double, ByVal knode() As Integer)
        '要素表面のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ie･･･要素番号
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        'knode()･･･節点の順位
        Dim i, ii, ip, j, kmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            kmax = 8
        End If
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        '    XG(numg) = 0
        '    YG(numg) = 0
        '    ZG(numg) = 0
        '    For ip = 1 To 8
        '        ii = knode(ip)
        '        XG(numg) = XG(numg) + XE(ii) * Shape(ii)
        '        YG(numg) = YG(numg) + YE(ii) * Shape(ii)
        '        ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
        '    Next ip
        For ip = 1 To kmax
            ii = knode(ip)
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ip

        '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの配列に、面積分で使用するための値を保存
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)

    End Sub

    Sub KHSolver(ByRef k As Integer, ByVal kmax As Integer, _
    ByRef s0 As Single, ByRef sfm As Single, ByRef s As Single, _
    ByRef t As Single, ByRef u As Single, ByVal xc As Single, _
    ByVal yc As Single, ByVal zc As Single)
        'KH法による最適化ｿﾙﾊﾞｰ
        'k･･･繰返し数
        'kmax･･･最大繰返し数（計算打切り）
        's0･･･初期の関数値
        'sfm･･･最適化した最終関数値
        's,t,u･･･要素の局所座標
        'xc,yc,zc･･･現在位置の座標
        Dim i, ichange, j, knew, nx As Integer
        Dim b1, b2, dx, sf, xb, xm As Single

        nx = 3              '変数の数
        Xvar(1) = s : Xvar(2) = t : Xvar(3) = u
        For i = 1 To nx
            Dratio(i) = 0.1
            EpsX(i) = 0.0001
        Next i

        Dim xmbuf(nx) As Single   '変数の前回中心値Ｘ
        Dim dxbuf(nx) As Single   '変数の前回変化ΔＸ

        k = 0
        Call CalFunc(s0, s, t, u, xc, yc, zc) '初期の関数値計算
        sfm = s0

        Do
            knew = 0    '変数の中心値Ｘと変化ΔＸの更新有無
            k = k + 1   '繰返し回数加算
            For i = 1 To nx      '各変数毎の計算
                xb = Xvar(i)
                xm = xb
                dx = xb * Dratio(i)
                If Math.Abs(dx) < EpsX(i) Then  'dxが0になるのを防止
                    dx = EpsX(i)
                End If
                If k = 1 Or xm <> xmbuf(i) Or dx <> dxbuf(i) Then
                    knew = 1
                End If
                xmbuf(i) = xm
                dxbuf(i) = dx

                ichange = 0         '各変数での改善有無
                For j = 1 To 2      '変数値の±ΔＸで改善比較
                    If j = 1 Then
                        Xvar(i) = xb - dx
                    Else
                        Xvar(i) = xb + dx
                    End If
                    Call CalFunc(sf, Xvar(1), Xvar(2), Xvar(3), xc, yc, zc)
                    '関数値計算
                    If sf < sfm Then        '最小化
                        '                If sf > sfm Then       '最大化
                        ichange = 1
                        sfm = sf
                        xm = Xvar(i)
                    End If
                Next j
                Xvar(i) = xm
                If ichange > 0 Then     '改善有りの場合はΔＸ比率を増倍
                    Dratio(i) = CInc * Dratio(i)
                Else        '改善なしの場合はΔＸ比率を低減
                    b1 = Dratio(i)
                    Dratio(i) = CRed * b1
                    b2 = Math.Abs(Xvar(i) * Dratio(i))

                    'ΔＸは最小変化量以上とする
                    If b2 < EpsX(i) Then
                        Dratio(i) = b1
                    End If
                End If
            Next i
        Loop While k < kmax And knew = 1
        s = Xvar(1)
        t = Xvar(2)
        u = Xvar(3)

    End Sub

    Function KSideLine(ByVal ib As Integer, ByVal kb As Integer, _
ByVal kk As Integer) As Integer
        '辺番号の照合
        'ib･･･要素番号
        'kb･･･面番号
        'kk･･･面の辺番号
        Dim ksb As Integer

        If KindElem = 0 Then    '六面体
            Select Case kb
                Case 1
                    ksb = kk
                Case 2
                    ksb = 13 - kk
                Case 3
                    Select Case kk
                        Case 1
                            ksb = 5
                        Case 2
                            ksb = 9
                        Case 3
                            ksb = 6
                        Case 4
                            ksb = 1
                    End Select
                Case 4
                    Select Case kk
                        Case 1
                            ksb = 7
                        Case 2
                            ksb = 11
                        Case 3
                            ksb = 8
                        Case 4
                            ksb = 3
                    End Select
                Case 5
                    Select Case kk
                        Case 1
                            ksb = 6
                        Case 2
                            ksb = 10
                        Case 3
                            ksb = 7
                        Case 4
                            ksb = 2
                    End Select
                Case 6
                    Select Case kk
                        Case 1
                            ksb = 8
                        Case 2
                            ksb = 12
                        Case 3
                            ksb = 5
                        Case 4
                            ksb = 4
                    End Select
            End Select
            KSideLine = 12 * (ib - 1) + ksb
        End If

    End Function

    Sub MiniSort2(ByVal kbuf() As Integer, ByRef nsort() As Integer, ByVal work() As Single, _
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ﾐﾆｿｰﾄ2
        '同じ変数値の場合に、第2座標値でソート
        'kind･･･主ｿｰﾄの座標軸番号（1:X軸、2:Y軸）
        'nsort()･･･元の変数番号のｿｰﾄ順番号
        'work()･･･ｿｰﾄ変数値
        'xbuf(),ybuf()･･･節点の座標
        Dim i, j, jb, jx, k, k1, k2, kx, nb, n1, n2, nx As Integer
        Dim b1 As Single
        Dim nsbuf(10), nsort2(10), nsort3(10) As Integer
        Dim work2(10), work3(10) As Single

        i = 1
        Do
            If work(i) = work(i + 1) Then
                b1 = work(i)
                n1 = i
                For j = i + 1 To Npoint
                    If b1 = work(j) Then
                        n2 = j
                    Else
                        Exit For
                    End If
                Next j
                nx = n2 - n1 + 1
                ReDim nsort2(nx), work2(nx)
                jb = 0
                For j = n1 To n2
                    jb = jb + 1
                    nsort2(jb) = jb
                    nb = nsort(j)
                    Select Case kbuf(2)
                        Case 1
                            work2(jb) = xbuf(nb)
                        Case 2
                            work2(jb) = ybuf(nb)
                        Case 3
                            work2(jb) = zbuf(nb)
                    End Select
                Next j
                Call ShellSortStd(nx, nsort2, work2)    'ｼｪｯﾙｿｰﾄ基本

                k = 1
                Do
                    If work2(k) = work2(k + 1) Then
                        b1 = work2(k)
                        k1 = k
                        For j = k + 1 To nx
                            If b1 = work2(j) Then
                                k2 = j
                            Else
                                Exit For
                            End If
                        Next j
                        kx = k2 - k1 + 1
                        ReDim nsort3(kx), work3(kx), nsbuf(kx)
                        jb = 0
                        For j = k1 To k2
                            jb = jb + 1
                            nsbuf(jb) = nsort2(j)
                            nsort3(jb) = jb
                            nb = nsort(n1 + nsort2(j) - 1)
                            Select Case kbuf(3)
                                Case 1
                                    work3(jb) = xbuf(nb)
                                Case 2
                                    work3(jb) = ybuf(nb)
                                Case 3
                                    work3(jb) = zbuf(nb)
                            End Select
                        Next j
                        Call ShellSortStd(kx, nsort3, work3)    'ｼｪｯﾙｿｰﾄ基本
                        For j = 1 To kx
                            jb = k1 + j - 1
                            nsort2(jb) = nsbuf(nsort3(j))
                        Next j
                        k = k2 + 1
                    Else
                        k = k + 1
                    End If
                Loop While k < nx

                For j = 1 To nx
                    jb = n1 + nsort2(j) - 1
                    nsort2(j) = nsort(jb)
                Next j
                For j = 1 To nx
                    jx = n1 + j - 1
                    nsort(jx) = nsort2(j)
                Next j
                i = n2 + 1
            Else
                i = i + 1
            End If
        Loop While i < Npoint

    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
    ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MouseCalcS(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            xb = (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
        Else
            xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
            yb = (Yupper - ya) / XYscale + YaxisMin
        End If

    End Sub

    Sub MousePointSearch(ByRef kp As Integer, ByVal xb As Single, _
    ByVal yb As Single, ByRef xmb() As Single, ByRef ymb() As Single)
        'ﾏｳｽ点の節点の探索
        'kp･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点の作画座標位置
        'xmb(),ymb()･･･節点のｽｸﾘｰﾝ座標
        Dim ie, k, kb, kk, kmax, nb As Integer
        Dim b1, b2, b3, rlen2, x2, y2 As Single
        Dim kbuf(Npoint) As Integer

        If KindElem = 0 Then
            kmax = 8
        End If
        kp = 0
        For nb = NSdraw To 1 Step -1
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
            End If
            For k = 1 To kmax
                kk = KsurfPoint(k)
                If kbuf(kk) = 0 Then
                    kbuf(kk) = 1
                    x2 = xmb(kk)
                    y2 = ymb(kk)
                    'Call XYCalc(x2, y2)     'X,Yの作画座標計算
                    b1 = x2 - xb
                    b2 = y2 - yb
                    b3 = b1 * b1 + b2 * b2
                    If kp = 0 Or b3 < rlen2 Then
                        kp = kk
                        rlen2 = b3
                    End If
                End If
            Next k
        Next nb

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\FEM3VRMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            KdataBackup = Val(sr.ReadLine)
            Kscreen = Val(sr.ReadLine)
            KcomSide = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\FEM3VRMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(KdataBackup)
        sw.WriteLine(Kscreen)
        sw.WriteLine(KcomSide)
        sw.Close()

    End Sub

    Sub OuterReset(ByVal k1 As Integer, ByVal k2 As Integer)
        '外郭面の再設定
        'k1,k2･･･開始・終了要素番号
        Dim ie, k, kmax As Integer

        If KindElem = 0 Then
            kmax = 6
        End If
        For ie = k1 To k2
            For k = 1 To kmax
                If KSelem(ie, k) < 0 Then
                    KSelem(ie, k) = 0
                End If
            Next k
        Next ie
    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i) + 0.001
            xydraw(i - 1).Y = Ydraw(i) + 0.001
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, ByVal rlenscreen As Single, _
    ByRef xvbuf() As Single, ByRef yvbuf() As Single, ByRef zvbuf() As Single, _
    ByRef xsbuf() As Single, ByRef ysbuf() As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        'xvbuf(),yvbuf(),zvbuf()･･･視線座標のX,Y,Z
        'xsbuf(),ysbuf()･･･ｽｸﾘｰﾝ座標のX,Y
        Dim ib, ie, j, nmax As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer

        If KindElem = 0 Then
            nmax = 20
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / zvbuf(ib)
                        xsbuf(ib) = b1 * xvbuf(ib)
                        ysbuf(ib) = b1 * (yvbuf(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PlaneCutElem()
        '面切断の要素選択
        Dim ic, ie, j, k, nmax As Integer
        Dim b1, std As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        If KE3DCut(2) > 0 Then
            std = VE3DCut(1) - VE3DCut(2)
        Else
            std = VE3DCut(1) + VE3DCut(2)
        End If

        For ie = 1 To Nelem
            If KdrawElem(ie) > 0 Then
                ic = 0
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    Select Case KE3DCut(1)
                        Case 1
                            b1 = Xdata(k)
                        Case 2
                            b1 = Ydata(k)
                        Case 3
                            b1 = Zdata(k)
                    End Select
                    If KE3DCut(2) > 0 Then
                        If b1 < std Then
                            ic = 1
                        End If
                    Else
                        If b1 > std Then
                            ic = 1
                        End If
                    End If
                    If ic > 0 Then
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PnumImproveNew(ByRef mb As Integer, ByVal mbx As Integer,
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        '節点順序の改良
        'mb･･･改良後のﾊﾞﾝﾄﾞ幅
        'mbx･･･改良前のﾊﾞﾝﾄﾞ幅
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ic, ie, k, k1, k2, mbb As Integer
        Dim rlenb As Single
        Dim kbuf(3), kbufb(3), kcbuf(Npoint) As Integer
        Dim rlbuf(3) As Single

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie
        Call AxisCheck(1, Nelem, rlenb, AxisMax, AxisMin, xbuf, ybuf, zbuf)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        mbb = 0
        For k = 1 To 3
            kbuf(1) = k
            Select Case k
                Case 1
                    k1 = 2
                    k2 = 3
                Case 2
                    k1 = 3
                    k2 = 1
                Case 3
                    k1 = 1
                    k2 = 2
            End Select
            rlbuf(k1) = AxisMax(k1) - AxisMin(k1)
            rlbuf(k2) = AxisMax(k2) - AxisMin(k2)
            If rlbuf(k1) > rlbuf(k2) Then
                kbuf(2) = k1
                kbuf(3) = k2
            Else
                kbuf(2) = k2
                kbuf(3) = k1
            End If

            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            ic = 0
            If k = 1 Or mb < mbb Then
                ic = 1
            End If
            If ic > 0 Then
                For i = 1 To 3
                    kbufb(i) = kbuf(i)
                Next i
                mbb = mb
            End If
        Next k

        If mbb < mbx Then
            For i = 1 To 3
                kbuf(i) = kbufb(i)
            Next i
            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            For i = 1 To Npoint
                Kcomp(i) = kcbuf(i)
            Next i
        Else
            mb = mbx
        End If

        For i = 1 To Npoint
            ib = Kcomp(i)
            KRcomp(ib) = i
        Next i

    End Sub

    Sub ResFileScan(ByVal ii As Integer)
        '結果ﾌｧｲﾙの読み飛ばし
        'ii･･･ﾌｧｲﾙ番号
        Dim aa$
        Dim i As Integer

        aa$ = ""
        Do
            Input(ii, aa$)
            If aa$ = "＜変位データ＞" Then
                Exit Do
            End If
        Loop
        Input(ii, aa$)
        For i = 1 To Npoint
            aa$ = LineInput(ii)
        Next i
        Input(ii, aa$) : Input(ii, aa$)
        For i = 1 To Npoint
            aa$ = LineInput(ii)
        Next i

    End Sub

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundPrep2(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備2（数値の最下位を基準）
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim a1$, aa$
        Dim k1, k2 As Integer

        aa$ = LTrim(Str(zb))
        k1 = Len(aa$)
        k2 = InStr(aa$, ".")
        If k2 > 0 Then
            kind = 1
            nb = k1 - k2
        Else
            kind = 0
            nb = 0
            Do
                nb = nb + 1
                a1$ = aa.Substring(k1 - nb, 1)
                If a1$ <> "0" Then
                    Exit Do
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub ShapeFunc(ByVal kind As Integer, ByVal s As Double, ByVal t As Double, _
    ByVal u As Double)
        '形状関数の計算
        'kind･･･計算区分（0:基本、1:偏微分込み）
        's･･･要素のξ座標
        't･･･要素のη座標
        'u･･･要素のζ座標
        Dim s1, s1m, s2m, t1, t1m, t2m, u1, u1m, u2m As Double
        s1 = 1 + s
        s1m = 1 - s
        s2m = 1 - s * s
        t1 = 1 + t
        t1m = 1 - t
        t2m = 1 - t * t
        u1 = 1 + u
        u1m = 1 - u
        u2m = 1 - u * u
        Shape(1) = s1m * t1m * u1 * (-s - t + u - 2) / 8
        Shape(2) = s2m * t1m * u1 / 4
        Shape(3) = s1 * t1m * u1 * (s - t + u - 2) / 8
        Shape(4) = s1 * t2m * u1 / 4
        Shape(5) = s1 * t1 * u1 * (s + t + u - 2) / 8
        Shape(6) = s2m * t1 * u1 / 4
        Shape(7) = s1m * t1 * u1 * (-s + t + u - 2) / 8
        Shape(8) = s1m * t2m * u1 / 4
        Shape(9) = s1m * t1m * u2m / 4
        Shape(10) = s1 * t1m * u2m / 4
        Shape(11) = s1 * t1 * u2m / 4
        Shape(12) = s1m * t1 * u2m / 4
        Shape(13) = s1m * t1m * u1m * (-s - t - u - 2) / 8
        Shape(14) = s2m * t1m * u1m / 4
        Shape(15) = s1 * t1m * u1m * (s - t - u - 2) / 8
        Shape(16) = s1 * t2m * u1m / 4
        Shape(17) = s1 * t1 * u1m * (s + t - u - 2) / 8
        Shape(18) = s2m * t1 * u1m / 4
        Shape(19) = s1m * t1 * u1m * (-s + t - u - 2) / 8
        Shape(20) = s1m * t2m * u1m / 4

        If kind > 0 Then
            Ds(1) = t1m * u1 * (s + t - u + 1 + s) / 8
            Ds(2) = -t1m * u1 * s / 2
            Ds(3) = t1m * u1 * (s - t + u - 1 + s) / 8
            Ds(4) = t2m * u1 / 4
            Ds(5) = t1 * u1 * (s + t + u - 1 + s) / 8
            Ds(6) = -t1 * u1 * s / 2
            Ds(7) = t1 * u1 * (s - t - u + 1 + s) / 8
            Ds(8) = -t2m * u1 / 4
            Ds(9) = -t1m * u2m / 4
            Ds(10) = t1m * u2m / 4
            Ds(11) = t1 * u2m / 4
            Ds(12) = -t1 * u2m / 4
            Ds(13) = t1m * u1m * (s + t + u + 1 + s) / 8
            Ds(14) = -t1m * u1m * s / 2
            Ds(15) = t1m * u1m * (s - t - u - 1 + s) / 8
            Ds(16) = t2m * u1m / 4
            Ds(17) = t1 * u1m * (s + t - u - 1 + s) / 8
            Ds(18) = -t1 * u1m * s / 2
            Ds(19) = t1 * u1m * (s - t + u + 1 + s) / 8
            Ds(20) = -t2m * u1m / 4
            Dt(1) = s1m * u1 * (s + t - u + 1 + t) / 8
            Dt(2) = -s2m * u1 / 4
            Dt(3) = s1 * u1 * (-s + t - u + 1 + t) / 8
            Dt(4) = -s1 * u1 * t / 2
            Dt(5) = s1 * u1 * (s + t + u - 1 + t) / 8
            Dt(6) = s2m * u1 / 4
            Dt(7) = s1m * u1 * (-s + t + u - 1 + t) / 8
            Dt(8) = -s1m * u1 * t / 2
            Dt(9) = -s1m * u2m / 4
            Dt(10) = -s1 * u2m / 4
            Dt(11) = s1 * u2m / 4
            Dt(12) = s1m * u2m / 4
            Dt(13) = s1m * u1m * (s + t + u + 1 + t) / 8
            Dt(14) = -s2m * u1m / 4
            Dt(15) = s1 * u1m * (-s + t + u + 1 + t) / 8
            Dt(16) = -s1 * u1m * t / 2
            Dt(17) = s1 * u1m * (s + t - u - 1 + t) / 8
            Dt(18) = s2m * u1m / 4
            Dt(19) = s1m * u1m * (-s + t - u - 1 + t) / 8
            Dt(20) = -s1m * u1m * t / 2
            Du(1) = s1m * t1m * (-s - t + u - 1 + u) / 8
            Du(2) = s2m * t1m / 4
            Du(3) = s1 * t1m * (s - t + u - 1 + u) / 8
            Du(4) = s1 * t2m / 4
            Du(5) = s1 * t1 * (s + t + u - 1 + u) / 8
            Du(6) = s2m * t1 / 4
            Du(7) = s1m * t1 * (-s + t + u - 1 + u) / 8
            Du(8) = s1m * t2m / 4
            Du(9) = -s1m * t1m * u / 2
            Du(10) = -s1 * t1m * u / 2
            Du(11) = -s1 * t1 * u / 2
            Du(12) = -s1m * t1 * u / 2
            Du(13) = s1m * t1m * (s + t + u + 1 + u) / 8
            Du(14) = -s2m * t1m / 4
            Du(15) = s1 * t1m * (-s + t + u + 1 + u) / 8
            Du(16) = -s1 * t2m / 4
            Du(17) = s1 * t1 * (-s - t + u + 1 + u) / 8
            Du(18) = -s2m * t1 / 4
            Du(19) = s1m * t1 * (s - t + u + 1 + u) / 8
            Du(20) = -s1m * t2m / 4
        End If
    End Sub

    Sub ShellSort2(ByVal kbuf() As Integer, ByRef kcbuf() As Integer, _
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ｼｪﾙｿｰﾄ
        Dim i As Integer
        Dim nsort(Npoint) As Integer
        Dim work(Npoint) As Single

        Select Case kbuf(1)
            Case 1
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Xdata(i)
                Next i
            Case 2
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Ydata(i)
                Next i
            Case 3
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Zdata(i)
                Next i
        End Select

        Call ShellSortStd(Npoint, nsort, work)    'ｼｪｯﾙｿｰﾄ基本

        Call MiniSort2(kbuf, nsort, work, xbuf, ybuf, zbuf)    'ﾐﾆｿｰﾄ

        For i = 1 To Npoint
            kcbuf(nsort(i)) = i
        Next i

    End Sub

    Sub ShellSortStd(ByVal nx As Integer, ByRef nsort() As Integer, ByRef work() As Single)
        'ｼｪﾙｿｰﾄ基本
        'nx･･･変数の数
        'nsort()･･ｿｰﾄ順の元変数の番号
        'work()･･･ｿｰﾄ順の変数値
        Dim i, ib, j, m As Integer
        Dim varb As Single

        m = nx
        Do While m > 1
            m = m \ 2
            For j = m + 1 To nx
                varb = work(j)
                ib = nsort(j)
                i = j - m
                Do While work(i) >= varb
                    work(i + m) = work(i)
                    nsort(i + m) = nsort(i)
                    i = i - m
                    If i <= 0 Then
                        Exit Do
                    End If
                Loop
                work(i + m) = varb
                nsort(i + m) = ib
            Next j
        Loop

    End Sub

    Sub SurfaceCond(ByVal kb As Integer, ByRef kind As Integer, _
        ByRef kbuf() As Integer, ByRef gpos() As Single)
        '境界条件用の面設定
        'kb･･･面番号
        'kind･･･面分類（1:面1と2、2:面3と4、3:面5と6）
        'kbuf()･･･面節点の基本番号
        'gpos()･･･積分点のξηζ座標
        Dim i As Integer
        Select Case kb
            Case 1
                kind = 1
                gpos(3) = 1
                For i = 1 To 8
                    kbuf(i) = i
                Next i
            Case 2
                kind = 1
                gpos(3) = -1
                kbuf(1) = 13
                kbuf(2) = 20
                kbuf(3) = 19
                kbuf(4) = 18
                kbuf(5) = 17
                kbuf(6) = 16
                kbuf(7) = 15
                kbuf(8) = 14
            Case 3
                kind = 2
                gpos(2) = -1
                kbuf(1) = 1
                kbuf(2) = 9
                kbuf(3) = 13
                kbuf(4) = 14
                kbuf(5) = 15
                kbuf(6) = 10
                kbuf(7) = 3
                kbuf(8) = 2
            Case 4
                kind = 2
                gpos(2) = 1
                kbuf(1) = 5
                kbuf(2) = 11
                kbuf(3) = 17
                kbuf(4) = 18
                kbuf(5) = 19
                kbuf(6) = 12
                kbuf(7) = 7
                kbuf(8) = 6
            Case 5
                kind = 3
                gpos(1) = 1
                kbuf(1) = 3
                kbuf(2) = 10
                kbuf(3) = 15
                kbuf(4) = 16
                kbuf(5) = 17
                kbuf(6) = 11
                kbuf(7) = 5
                kbuf(8) = 4
            Case 6
                kind = 3
                gpos(1) = -1
                kbuf(1) = 7
                kbuf(2) = 12
                kbuf(3) = 19
                kbuf(4) = 20
                kbuf(5) = 13
                kbuf(6) = 9
                kbuf(7) = 1
                kbuf(8) = 8
        End Select

    End Sub

    Sub SurfaceDrawC(ByRef gr As Graphics, ByVal k1 As Integer,
    ByVal k2 As Integer, ByVal kind As Integer)
        '面の作画（材料別色分け）
        'k1,k2･･･要素の開始・終了番号
        'kind･･･面外周の作画有無（0:なし、1:有り）
        Dim aa$
        Dim ie, j, jmax, k, kb, kk, kmax, nb, nmax As Integer
        Dim xb, xd, yb, yd As Single
        Dim fnt As Font = New Font("ＭＳ ゴシック", KnumSize)
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", KnumSb, FontStyle.Bold)

        If KindElem = 0 Then
            jmax = 6
            kmax = 8
            nmax = 20
        End If
        For nb = 1 To NSdraw
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint) '面の設定
            End If
            For k = 1 To kmax
                kk = KsurfPoint(k)
                xd = Xscreen(kk)
                yd = Yscreen(kk)
                Call XYCalc(xd, yd)     'X,Yの作画座標計算
                Xdraw(k) = xd
                Ydraw(k) = yd
            Next k
            kk = (NumMat(ie) - 1) Mod 7
            Select Case kk
                Case 0
                    Call PaintExe(gr, kmax, Color.White, kind)
                Case 1
                    Call PaintExe(gr, kmax, Color.LightBlue, kind)
                Case 2
                    Call PaintExe(gr, kmax, Color.LightGreen, kind)
                Case 3
                    Call PaintExe(gr, kmax, Color.LightCyan, kind)
                Case 4
                    Call PaintExe(gr, kmax, Color.LightPink, kind)
                Case 5
                    Call PaintExe(gr, kmax, Color.LightYellow, kind)
                Case 6
                    Call PaintExe(gr, kmax, Color.LightGray, kind)
            End Select
            If KnumDisp > 0 Then
                xb = 0 : yb = 0
                For k = 1 To kmax
                    xb = xb + Xdraw(k)
                    yb = yb + Ydraw(k)
                    If KdispPNum > 0 Then
                        kk = KsurfPoint(k)
                        xd = Xdraw(k)
                        yd = Ydraw(k)
                        aa$ = LTrim$(Str$(kk))
                        gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                    End If
                Next k
                If KdispSNum > 0 Then
                    xd = xb / kmax
                    yd = yb / kmax
                    aa$ = "(" + LTrim$(Str$(kb)) + ")"
                    gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                End If
                If KdispENum > 0 Then
                    xb = 0 : yb = 0
                    For j = 1 To nmax
                        kb = NNelem(ie, j)
                        xd = Xscreen(kb)
                        yd = Yscreen(kb)
                        Call XYCalc(xd, yd) 'X,Yの作画座標計算
                        xb = xb + xd
                        yb = yb + yd
                    Next j
                    xd = xb / nmax
                    yd = yb / nmax
                    aa$ = LTrim$(Str$(ie))
                    gr.DrawString(aa$, fnt2, Brushes.Black, xd, yd)
                End If
            End If
        Next nb

        If k1 <= KmouseElem And KmouseElem <= k2 Then
            j = 0
            For nb = 1 To NSdraw
                ie = KEorder(nb)
                kb = KSorder(nb)
                If ie = KmouseElem And kb = KmouseSurf Then
                    j = 1
                    Exit For
                End If
            Next nb
            If j > 0 Then
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
                End If
                For k = 1 To kmax
                    kk = KsurfPoint(k)
                    xd = Xscreen(kk)
                    yd = Yscreen(kk)
                    Call XYCalc(xd, yd)     'X,Yの作画座標計算
                    Xdraw(k) = xd
                    Ydraw(k) = yd
                Next k
                xb = 0 : yb = 0
                For k = 1 To kmax
                    xb = xb + Xdraw(k)
                    yb = yb + Ydraw(k)
                    kk = KsurfPoint(k)
                    xd = Xdraw(k)
                    yd = Ydraw(k)
                    aa$ = LTrim$(Str$(kk))
                    gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                Next k
                xd = xb / kmax
                yd = yb / kmax
                aa$ = "(" + LTrim$(Str$(kb)) + ")"
                gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                xb = 0 : yb = 0
                For j = 1 To nmax
                    kb = NNelem(ie, j)
                    xd = Xscreen(kb)
                    yd = Yscreen(kb)
                    Call XYCalc(xd, yd) 'X,Yの作画座標計算
                    xb = xb + xd
                    yb = yb + yd
                Next j
                xd = xb / nmax
                yd = yb / nmax
                aa$ = LTrim$(Str$(ie))
                gr.DrawString(aa$, fnt2, Brushes.Black, xd, yd)
            End If
        End If

    End Sub

    Sub SurfaceSet(ByVal ib As Integer, ByVal kb As Integer, _
       ByRef kbuf() As Integer)
        '面の設定
        'ib･･･要素番号
        'kb･･･面番号
        'kbuf()･･･面の節点番号
        Select Case kb
            Case 1
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 2)
                kbuf(3) = NNelem(ib, 3)
                kbuf(4) = NNelem(ib, 4)
                kbuf(5) = NNelem(ib, 5)
                kbuf(6) = NNelem(ib, 6)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 8)
            Case 2
                kbuf(1) = NNelem(ib, 13)
                kbuf(2) = NNelem(ib, 20)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 16)
                kbuf(7) = NNelem(ib, 15)
                kbuf(8) = NNelem(ib, 14)
            Case 3
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 9)
                kbuf(3) = NNelem(ib, 13)
                kbuf(4) = NNelem(ib, 14)
                kbuf(5) = NNelem(ib, 15)
                kbuf(6) = NNelem(ib, 10)
                kbuf(7) = NNelem(ib, 3)
                kbuf(8) = NNelem(ib, 2)
            Case 4
                kbuf(1) = NNelem(ib, 5)
                kbuf(2) = NNelem(ib, 11)
                kbuf(3) = NNelem(ib, 17)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 19)
                kbuf(6) = NNelem(ib, 12)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 6)
            Case 5
                kbuf(1) = NNelem(ib, 3)
                kbuf(2) = NNelem(ib, 10)
                kbuf(3) = NNelem(ib, 15)
                kbuf(4) = NNelem(ib, 16)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 11)
                kbuf(7) = NNelem(ib, 5)
                kbuf(8) = NNelem(ib, 4)
            Case 6
                kbuf(1) = NNelem(ib, 7)
                kbuf(2) = NNelem(ib, 12)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 20)
                kbuf(5) = NNelem(ib, 13)
                kbuf(6) = NNelem(ib, 9)
                kbuf(7) = NNelem(ib, 1)
                kbuf(8) = NNelem(ib, 8)
        End Select

    End Sub

    Sub SurfaceVector(ByVal planevec() As Single, ByVal xbuf() As Single, _
        ByVal ybuf() As Single, ByVal zbuf() As Single)
        '面の法線ﾍﾞｸﾄﾙの計算
        'planevec()･･･面の法線ﾍﾞｸﾄﾙの各方向余弦
        'xbuf(),ybuf(),zbuf()･･･3点のX,Y,Z座標
        Dim b1, b2, b3, b4 As Single
        b1 = (ybuf(2) - ybuf(1)) * (zbuf(3) - zbuf(1))
        b1 = b1 - (zbuf(2) - zbuf(1)) * (ybuf(3) - ybuf(1))
        b2 = (zbuf(2) - zbuf(1)) * (xbuf(3) - xbuf(1))
        b2 = b2 - (xbuf(2) - xbuf(1)) * (zbuf(3) - zbuf(1))
        b3 = (xbuf(2) - xbuf(1)) * (ybuf(3) - ybuf(1))
        b3 = b3 - (ybuf(2) - ybuf(1)) * (xbuf(3) - xbuf(1))
        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
        planevec(1) = b1 / b4
        planevec(2) = b2 / b4
        planevec(3) = b3 / b4

    End Sub

    Sub SurfSort(ByVal k1 As Integer, ByVal k2 As Integer)
        '作画面のｿｰﾃｨﾝｸﾞ
        'k1,k2･･･開始・終了要素番号
        Dim i1, i2, ic, ie, ii, ik1, ik2, j, k, kmax, nmax, nb As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
        End If
        NSdraw = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        NSdraw = NSdraw + 1
                    End If
                Next k
            End If
        Next ie
        ReDim KEorder(NSdraw), KSorder(NSdraw)
        nb = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        nb = nb + 1
                        KEorder(nb) = ie
                        KSorder(nb) = k
                    End If
                Next k
            End If
        Next ie

        For ii = 1 To NSdraw - 1
            i1 = KEorder(ii)
            ik1 = KSorder(ii)
            If KindElem = 0 Then
                Call SurfaceSet(i1, ik1, KsurfPoint)    '面の設定
            End If

            For j = ii + 1 To NSdraw
                i2 = KEorder(j)
                ik2 = KSorder(j)
                If KindElem = 0 Then
                    Call SurfaceSet(i2, ik2, kbuf)      '面の設定
                End If

                Call DistComp(ic, kbuf)    '距離比較
                If ic = 0 Then
                    KEorder(ii) = i2
                    KSorder(ii) = ik2
                    KEorder(j) = i1
                    KSorder(j) = ik1
                    i1 = i2
                    ik1 = ik2
                    For k = 1 To nmax
                        KsurfPoint(k) = kbuf(k)
                    Next k
                End If
            Next j
        Next ii

    End Sub

    Sub TimCal(ByRef ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalA(ByVal xs As Single, ByVal ys As Single, _
        ByVal zs As Single, ByVal xr As Single, ByVal yr As Single, _
        ByVal zr As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        '先にZ,Y,X軸回りの回転、最後に座標軸方向の平行移動
        'xs,ys,zs･･･X,Y,Z方向移動量
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i
        buf1(1, 4) = xs
        buf1(2, 4) = ys
        buf1(3, 4) = zs

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + TransMat(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
        ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\FEM3VibRes.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fstruct$ = sr.ReadLine
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\FEM3VibRes.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fstruct$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub TrSet(ByRef tr(,) As Single)
        '節点応力計算のTr行列の設定
        Dim i, j As Integer
        Dim abuf As Single

        For i = 1 To 2
            For j = 1 To 27
                If i = 1 Then
                    Select Case j
                        Case 1
                            abuf = -0.1255884
                            tr(1, 1) = abuf : tr(1, 9) = abuf : tr(1, 21) = abuf
                            tr(3, 3) = abuf : tr(3, 19) = abuf : tr(3, 27) = abuf
                            tr(5, 9) = abuf : tr(5, 21) = abuf : tr(5, 25) = abuf
                            tr(7, 3) = abuf : tr(7, 7) = abuf : tr(7, 27) = abuf
                            tr(13, 3) = abuf : tr(13, 7) = abuf : tr(13, 19) = abuf
                            tr(15, 1) = abuf : tr(15, 21) = abuf : tr(15, 25) = abuf
                            tr(17, 7) = abuf : tr(17, 19) = abuf : tr(17, 27) = abuf
                            tr(19, 1) = abuf : tr(19, 9) = abuf : tr(19, 25) = abuf
                        Case 2
                            abuf = -0.3124648
                            tr(1, 2) = abuf : tr(1, 6) = abuf : tr(1, 12) = abuf
                            tr(3, 12) = abuf : tr(3, 20) = abuf : tr(3, 24) = abuf
                            tr(5, 18) = abuf : tr(5, 24) = abuf : tr(5, 26) = abuf
                            tr(7, 6) = abuf : tr(7, 8) = abuf : tr(7, 18) = abuf
                            tr(13, 2) = abuf : tr(13, 4) = abuf : tr(13, 10) = abuf
                            tr(15, 10) = abuf : tr(15, 20) = abuf : tr(15, 22) = abuf
                            tr(17, 16) = abuf : tr(17, 22) = abuf : tr(17, 26) = abuf
                            tr(19, 4) = abuf : tr(19, 8) = abuf : tr(19, 16) = abuf
                        Case 3
                            abuf = 2.374992
                            tr(1, 3) = abuf : tr(3, 21) = abuf
                            tr(5, 27) = abuf : tr(7, 9) = abuf
                            tr(13, 1) = abuf : tr(15, 19) = abuf
                            tr(17, 25) = abuf : tr(19, 7) = abuf
                        Case 4
                            abuf = 0.3148157
                            tr(1, 4) = abuf : tr(1, 8) = abuf : tr(1, 10) = abuf
                            tr(1, 18) = abuf : tr(1, 20) = abuf : tr(1, 24) = abuf
                            tr(3, 2) = abuf : tr(3, 6) = abuf : tr(3, 10) = abuf
                            tr(3, 18) = abuf : tr(3, 22) = abuf : tr(3, 26) = abuf
                            tr(5, 6) = abuf : tr(5, 8) = abuf : tr(5, 12) = abuf
                            tr(5, 16) = abuf : tr(5, 20) = abuf : tr(5, 22) = abuf
                            tr(7, 2) = abuf : tr(7, 4) = abuf : tr(7, 12) = abuf
                            tr(7, 16) = abuf : tr(7, 24) = abuf : tr(7, 26) = abuf
                            tr(13, 6) = abuf : tr(13, 8) = abuf : tr(13, 12) = abuf
                            tr(13, 16) = abuf : tr(13, 20) = abuf : tr(13, 22) = abuf
                            tr(15, 2) = abuf : tr(15, 4) = abuf : tr(15, 12) = abuf
                            tr(15, 16) = abuf : tr(15, 24) = abuf : tr(15, 26) = abuf
                            tr(17, 4) = abuf : tr(17, 8) = abuf : tr(17, 10) = abuf
                            tr(17, 18) = abuf : tr(17, 20) = abuf : tr(17, 24) = abuf
                            tr(19, 2) = abuf : tr(19, 6) = abuf : tr(19, 10) = abuf
                            tr(19, 18) = abuf : tr(19, 22) = abuf : tr(19, 26) = abuf
                        Case 5
                            abuf = -0.4882397
                            tr(1, 5) = abuf : tr(1, 11) = abuf : tr(1, 15) = abuf
                            tr(3, 11) = abuf : tr(3, 15) = abuf : tr(3, 23) = abuf
                            tr(5, 15) = abuf : tr(5, 17) = abuf : tr(5, 23) = abuf
                            tr(7, 5) = abuf : tr(7, 15) = abuf : tr(7, 17) = abuf
                            tr(13, 5) = abuf : tr(13, 11) = abuf : tr(13, 13) = abuf
                            tr(15, 11) = abuf : tr(15, 13) = abuf : tr(15, 23) = abuf
                            tr(17, 13) = abuf : tr(17, 17) = abuf : tr(17, 23) = abuf
                            tr(19, 5) = abuf : tr(19, 13) = abuf : tr(19, 17) = abuf
                        Case 7
                            abuf = -0.1614496
                            tr(1, 7) = abuf : tr(1, 19) = abuf : tr(1, 27) = abuf
                            tr(3, 1) = abuf : tr(3, 9) = abuf : tr(3, 25) = abuf
                            tr(5, 3) = abuf : tr(5, 7) = abuf : tr(5, 19) = abuf
                            tr(7, 1) = abuf : tr(7, 21) = abuf : tr(7, 25) = abuf
                            tr(13, 9) = abuf : tr(13, 21) = abuf : tr(13, 25) = abuf
                            tr(15, 3) = abuf : tr(15, 7) = abuf : tr(15, 27) = abuf
                            tr(17, 1) = abuf : tr(17, 9) = abuf : tr(17, 21) = abuf
                            tr(19, 3) = abuf : tr(19, 19) = abuf : tr(19, 27) = abuf
                        Case 13
                            abuf = 0.2289803
                            tr(1, 13) = abuf : tr(1, 17) = abuf : tr(1, 23) = abuf
                            tr(3, 5) = abuf : tr(3, 13) = abuf : tr(3, 17) = abuf
                            tr(5, 5) = abuf : tr(5, 11) = abuf : tr(5, 13) = abuf
                            tr(7, 11) = abuf : tr(7, 13) = abuf : tr(7, 23) = abuf
                            tr(13, 15) = abuf : tr(13, 17) = abuf : tr(13, 23) = abuf
                            tr(15, 5) = abuf : tr(15, 15) = abuf : tr(15, 17) = abuf
                            tr(17, 5) = abuf : tr(17, 11) = abuf : tr(17, 15) = abuf
                            tr(19, 11) = abuf : tr(19, 15) = abuf : tr(19, 23) = abuf
                        Case 14
                            abuf = -0.2962959
                            tr(1, 14) = abuf : tr(3, 14) = abuf
                            tr(5, 14) = abuf : tr(7, 14) = abuf
                            tr(13, 14) = abuf : tr(15, 14) = abuf
                            tr(17, 14) = abuf : tr(19, 14) = abuf
                        Case 16
                            abuf = -0.16902
                            tr(1, 16) = abuf : tr(1, 22) = abuf : tr(1, 26) = abuf
                            tr(3, 4) = abuf : tr(3, 8) = abuf : tr(3, 16) = abuf
                            tr(5, 2) = abuf : tr(5, 4) = abuf : tr(5, 10) = abuf
                            tr(7, 10) = abuf : tr(7, 20) = abuf : tr(7, 22) = abuf
                            tr(13, 18) = abuf : tr(13, 24) = abuf : tr(13, 26) = abuf
                            tr(15, 6) = abuf : tr(15, 8) = abuf : tr(15, 18) = abuf
                            tr(17, 2) = abuf : tr(17, 6) = abuf : tr(17, 12) = abuf
                            tr(19, 12) = abuf : tr(19, 20) = abuf : tr(19, 24) = abuf
                        Case 25
                            abuf = 0.1157502
                            tr(1, 25) = abuf : tr(3, 7) = abuf
                            tr(5, 1) = abuf : tr(7, 19) = abuf
                            tr(13, 27) = abuf : tr(15, 9) = abuf
                            tr(17, 3) = abuf : tr(19, 21) = abuf
                    End Select
                Else
                    Select Case j
                        Case 1
                            abuf = 0.1111112
                            tr(2, 1) = abuf : tr(2, 9) = abuf
                            tr(2, 19) = abuf : tr(2, 27) = abuf
                            tr(4, 3) = abuf : tr(4, 9) = abuf
                            tr(4, 19) = abuf : tr(4, 25) = abuf
                            tr(6, 3) = abuf : tr(6, 21) = abuf
                            tr(6, 7) = abuf : tr(6, 25) = abuf
                            tr(8, 1) = abuf : tr(8, 7) = abuf
                            tr(8, 21) = abuf : tr(8, 27) = abuf
                            tr(9, 7) = abuf : tr(9, 9) = abuf
                            tr(9, 19) = abuf : tr(9, 21) = abuf
                            tr(10, 1) = abuf : tr(10, 3) = abuf
                            tr(10, 25) = abuf : tr(10, 27) = abuf
                            tr(11, 7) = abuf : tr(11, 9) = abuf
                            tr(11, 19) = abuf : tr(11, 21) = abuf
                            tr(12, 1) = abuf : tr(12, 3) = abuf
                            tr(12, 25) = abuf : tr(12, 27) = abuf
                            tr(14, 3) = abuf : tr(14, 21) = abuf
                            tr(14, 7) = abuf : tr(14, 25) = abuf
                            tr(16, 1) = abuf : tr(16, 7) = abuf
                            tr(16, 21) = abuf : tr(16, 27) = abuf
                            tr(18, 1) = abuf : tr(18, 19) = abuf
                            tr(18, 9) = abuf : tr(18, 27) = abuf
                            tr(20, 3) = abuf : tr(20, 9) = abuf
                            tr(20, 19) = abuf : tr(20, 25) = abuf
                        Case 2
                            abuf = -0.2707217
                            tr(2, 2) = abuf : tr(2, 20) = abuf
                            tr(2, 6) = abuf : tr(2, 24) = abuf
                            tr(4, 12) = abuf : tr(4, 18) = abuf
                            tr(4, 20) = abuf : tr(4, 26) = abuf
                            tr(6, 6) = abuf : tr(6, 24) = abuf
                            tr(6, 8) = abuf : tr(6, 26) = abuf
                            tr(8, 2) = abuf : tr(8, 8) = abuf
                            tr(8, 12) = abuf : tr(8, 18) = abuf
                            tr(9, 4) = abuf : tr(9, 6) = abuf
                            tr(9, 10) = abuf : tr(9, 12) = abuf
                            tr(10, 10) = abuf : tr(10, 12) = abuf
                            tr(10, 22) = abuf : tr(10, 24) = abuf
                            tr(11, 16) = abuf : tr(11, 18) = abuf
                            tr(11, 22) = abuf : tr(11, 24) = abuf
                            tr(12, 4) = abuf : tr(12, 6) = abuf
                            tr(12, 16) = abuf : tr(12, 18) = abuf
                            tr(14, 2) = abuf : tr(14, 20) = abuf
                            tr(14, 4) = abuf : tr(14, 22) = abuf
                            tr(16, 10) = abuf : tr(16, 16) = abuf
                            tr(16, 20) = abuf : tr(16, 26) = abuf
                            tr(18, 4) = abuf : tr(18, 22) = abuf
                            tr(18, 8) = abuf : tr(18, 26) = abuf
                            tr(20, 2) = abuf : tr(20, 8) = abuf
                            tr(20, 10) = abuf : tr(20, 16) = abuf
                        Case 3
                            abuf = 0.3262776
                            tr(2, 3) = abuf : tr(2, 21) = abuf
                            tr(4, 21) = abuf : tr(4, 27) = abuf
                            tr(6, 9) = abuf : tr(6, 27) = abuf
                            tr(8, 3) = abuf : tr(8, 9) = abuf
                            tr(9, 1) = abuf : tr(9, 3) = abuf
                            tr(10, 19) = abuf : tr(10, 21) = abuf
                            tr(11, 25) = abuf : tr(11, 27) = abuf
                            tr(12, 7) = abuf : tr(12, 9) = abuf
                            tr(14, 1) = abuf : tr(14, 19) = abuf
                            tr(16, 19) = abuf : tr(16, 25) = abuf
                            tr(18, 7) = abuf : tr(18, 25) = abuf
                            tr(20, 1) = abuf : tr(20, 7) = abuf
                        Case 4
                            abuf = 0.1596107
                            tr(2, 4) = abuf : tr(2, 22) = abuf
                            tr(2, 8) = abuf : tr(2, 26) = abuf
                            tr(4, 2) = abuf : tr(4, 8) = abuf
                            tr(4, 10) = abuf : tr(4, 16) = abuf
                            tr(6, 2) = abuf : tr(6, 20) = abuf
                            tr(6, 4) = abuf : tr(6, 22) = abuf
                            tr(8, 10) = abuf : tr(8, 16) = abuf
                            tr(8, 20) = abuf : tr(8, 26) = abuf
                            tr(9, 16) = abuf : tr(9, 18) = abuf
                            tr(9, 22) = abuf : tr(9, 24) = abuf
                            tr(10, 4) = abuf : tr(10, 6) = abuf
                            tr(10, 16) = abuf : tr(10, 18) = abuf
                            tr(11, 4) = abuf : tr(11, 6) = abuf
                            tr(11, 10) = abuf : tr(11, 12) = abuf
                            tr(12, 10) = abuf : tr(12, 12) = abuf
                            tr(12, 22) = abuf : tr(12, 24) = abuf
                            tr(14, 6) = abuf : tr(14, 24) = abuf
                            tr(14, 8) = abuf : tr(14, 26) = abuf
                            tr(16, 2) = abuf : tr(16, 8) = abuf
                            tr(16, 12) = abuf : tr(16, 18) = abuf
                            tr(18, 2) = abuf : tr(18, 20) = abuf
                            tr(18, 6) = abuf : tr(18, 24) = abuf
                            tr(20, 12) = abuf : tr(20, 18) = abuf
                            tr(20, 20) = abuf : tr(20, 26) = abuf
                        Case 5
                            abuf = -0.2222226
                            tr(2, 5) = abuf : tr(2, 23) = abuf
                            tr(4, 11) = abuf : tr(4, 17) = abuf
                            tr(6, 5) = abuf : tr(6, 23) = abuf
                            tr(8, 11) = abuf : tr(8, 17) = abuf
                            tr(9, 13) = abuf : tr(9, 15) = abuf
                            tr(10, 13) = abuf : tr(10, 15) = abuf
                            tr(11, 13) = abuf : tr(11, 15) = abuf
                            tr(12, 13) = abuf : tr(12, 15) = abuf
                            tr(14, 5) = abuf : tr(14, 23) = abuf
                            tr(16, 11) = abuf : tr(16, 17) = abuf
                            tr(18, 5) = abuf : tr(18, 23) = abuf
                            tr(20, 11) = abuf : tr(20, 17) = abuf
                        Case 7
                            abuf = -0.1040551
                            tr(2, 7) = abuf : tr(2, 25) = abuf
                            tr(4, 1) = abuf : tr(4, 7) = abuf
                            tr(6, 1) = abuf : tr(6, 19) = abuf
                            tr(8, 19) = abuf : tr(8, 25) = abuf
                            tr(9, 25) = abuf : tr(9, 27) = abuf
                            tr(10, 7) = abuf : tr(10, 9) = abuf
                            tr(11, 1) = abuf : tr(11, 3) = abuf
                            tr(12, 19) = abuf : tr(12, 21) = abuf
                            tr(14, 9) = abuf : tr(14, 27) = abuf
                            tr(16, 3) = abuf : tr(16, 9) = abuf
                            tr(18, 3) = abuf : tr(18, 21) = abuf
                            tr(20, 21) = abuf : tr(20, 27) = abuf
                        Case 10
                            abuf = -0.194445
                            tr(2, 10) = abuf : tr(2, 18) = abuf
                            tr(4, 6) = abuf : tr(4, 22) = abuf
                            tr(6, 12) = abuf : tr(6, 16) = abuf
                            tr(8, 4) = abuf : tr(8, 24) = abuf
                            tr(9, 8) = abuf : tr(9, 20) = abuf
                            tr(10, 2) = abuf : tr(10, 26) = abuf
                            tr(11, 8) = abuf : tr(11, 20) = abuf
                            tr(12, 2) = abuf : tr(12, 26) = abuf
                            tr(14, 12) = abuf : tr(14, 16) = abuf
                            tr(16, 4) = abuf : tr(16, 24) = abuf
                            tr(18, 10) = abuf : tr(18, 18) = abuf
                            tr(20, 6) = abuf : tr(20, 22) = abuf
                        Case 11
                            abuf = 0.05555563
                            tr(2, 11) = abuf : tr(2, 15) = abuf
                            tr(4, 15) = abuf : tr(4, 23) = abuf
                            tr(6, 15) = abuf : tr(6, 17) = abuf
                            tr(8, 5) = abuf : tr(8, 15) = abuf
                            tr(9, 5) = abuf : tr(9, 11) = abuf
                            tr(10, 11) = abuf : tr(10, 23) = abuf
                            tr(11, 17) = abuf : tr(11, 23) = abuf
                            tr(12, 5) = abuf : tr(12, 17) = abuf
                            tr(14, 11) = abuf : tr(14, 13) = abuf
                            tr(16, 13) = abuf : tr(16, 23) = abuf
                            tr(18, 13) = abuf : tr(18, 17) = abuf
                            tr(20, 5) = abuf : tr(20, 13) = abuf
                        Case 12
                            abuf = 1.284386
                            tr(2, 12) = abuf : tr(4, 24) = abuf
                            tr(6, 18) = abuf : tr(8, 6) = abuf
                            tr(9, 2) = abuf : tr(10, 20) = abuf
                            tr(11, 26) = abuf : tr(12, 8) = abuf
                            tr(14, 10) = abuf : tr(16, 22) = abuf
                            tr(18, 16) = abuf : tr(20, 4) = abuf
                        Case 13
                            abuf = 0.05555541
                            tr(2, 13) = abuf : tr(2, 17) = abuf
                            tr(4, 5) = abuf : tr(4, 13) = abuf
                            tr(6, 11) = abuf : tr(6, 13) = abuf
                            tr(8, 13) = abuf : tr(8, 23) = abuf
                            tr(9, 17) = abuf : tr(9, 23) = abuf
                            tr(10, 5) = abuf : tr(10, 17) = abuf
                            tr(11, 5) = abuf : tr(11, 11) = abuf
                            tr(12, 11) = abuf : tr(12, 23) = abuf
                            tr(14, 15) = abuf : tr(14, 17) = abuf
                            tr(16, 5) = abuf : tr(16, 15) = abuf
                            tr(18, 11) = abuf : tr(18, 15) = abuf
                            tr(20, 15) = abuf : tr(20, 23) = abuf
                        Case 14
                            abuf = -0.111111
                            tr(2, 14) = abuf : tr(4, 14) = abuf
                            tr(6, 14) = abuf : tr(8, 14) = abuf
                            tr(9, 14) = abuf : tr(10, 14) = abuf
                            tr(11, 14) = abuf : tr(12, 14) = abuf
                            tr(14, 14) = abuf : tr(16, 14) = abuf
                            tr(18, 14) = abuf : tr(20, 14) = abuf
                        Case 16
                            abuf = -0.006608076
                            tr(2, 16) = abuf : tr(4, 4) = abuf
                            tr(6, 10) = abuf : tr(8, 22) = abuf
                            tr(9, 26) = abuf : tr(10, 8) = abuf
                            tr(11, 2) = abuf : tr(12, 20) = abuf
                            tr(14, 18) = abuf : tr(16, 6) = abuf
                            tr(18, 12) = abuf : tr(20, 24) = abuf
                    End Select
                End If
            Next j
        Next i

    End Sub

    Sub UFileAddA()
        'UFileAの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileA(i) = UFileA(0) Then
                For j = i - 1 To 0 Step -1
                    UFileA(j + 1) = UFileA(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileA(i + 1) = UFileA(i)
            Next i
        End If
        UseFileAOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UFileAddB()
        'UFileBの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileB(i) = UFileB(0) Then
                For j = i - 1 To 0 Step -1
                    UFileB(j + 1) = UFileB(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileB(i + 1) = UFileB(i)
            Next i
        End If
        UseFileBOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileAIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM3VRUFileA.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileA(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileAOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM3VRUFileA.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileA(i) <> "" Then
                sw.WriteLine(UFileA(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub UseFileBIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM3VRUFileB.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileB(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileBOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FEM3VRUFileB.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileB(i) <> "" Then
                sw.WriteLine(UFileB(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

End Module
